require "Window"
require "GameLib"
require "Unit"


local StalkerResourceSystem = {}
local VILib = _G["GeminiPackages"]:GetPackage("VILib-1.2")

local sVersion = "1.2.0"
local arTextAligns = {
	"Left",
	"Center",
	"Right",
}
local tFonts = {
	{"CRB_Header24_O", "Header"},
	{"CRB_FloaterGigantic_O", "Floater"},
	{"CRB_Interface16_BO", "Interface"},
	{"CRB_Interface16_BBO", "Interface B"},
	{"CRB_Pixel_O", "Pixel"},
	{"Subtitle", "Subtitle"},
	{"CRB_Dialog_Heading_Huge", "Dialog"},
}
local tNoHighlight = {
	[1] = true,
	[2] = true,
	[16] = true,
}
local arPositionOnCharacterNames = {
	"Feet",
	"Head",
}
local tAnchors = {
	{sLabel = "Center", tPoints = {0.5, 0.5, 0.5, 0.5}, tOffsetMults = {-0.5, -0.5, 0.5, 0.5}},
	{sLabel = "Left", tPoints = {0, 0.5, 0, 0.5}, tOffsetMults = {0, -0.5, 1, 0.5}},
	{sLabel = "Top Left", tPoints = {0, 0, 0, 0}, tOffsetMults = {0, 0, 1, 1}},
	{sLabel = "Top", tPoints = {0.5, 0, 0.5, 0}, tOffsetMults = {-0.5, 0, 0.5, 1}},
	{sLabel = "Top Right", tPoints = {1, 0, 1, 0}, tOffsetMults = {-1, 0, 0, 1}},
	{sLabel = "Right", tPoints = {1, 0.5, 1, 0.5}, tOffsetMults = {-1, -0.5, 0, 0.5}},
	{sLabel = "Bottom Right", tPoints = {1, 1, 1, 1}, tOffsetMults = {-1, -1, 0, 0}},
	{sLabel = "Bottom", tPoints = {0.5, 1, 0.5, 1}, tOffsetMults = {-0.5, -1, 0.5, 0}},
	{sLabel = "Bottom Left", tPoints = {0, 1, 0, 1}, tOffsetMults = {0, -1, 1, 0}},
}
local tBorderOffsets = {
	{16, 16, -16, -16},
}

local tDefaultSettings = {
	tWindowPositions = {
		tMain = {},
		tMainOffsets = {
			{0, 140},
			{0, 0},
			{0, 0},
			{0, 0},
			{0, 0},
			{0, 0},
			{0, 0},
			{0, 0},
			{0, 0},
		},
		tMainCharOffsets = {0, 0},
		tOptions = {},
		tBarStyleOptions = {},
		tFontOptions = {},
		tMarkerSystem = {},
		nMainAnchor = 1,
	},
	tColors = {
		sText = "ffffffff",
		sBarTest = "ffff0808",
		bBarTest2x = false,
		sBackgroundBar = "ffffffff",
		bBackgroundBar2x = false,
		sBar = "ff4dd29c",
		bBar2x = false,
		sBarFlash = "ffffffff",
		sBorder = "ff000000",
		sBackground = "ff000000",
		sPattern = "ff8cf4d1",
		sGraphicBorder = "ffffffff",
		sBGFlash1 = "ffffffff",
		bBGFlash1 = false,
		tStances = {
			"ffff0808", -- Lethal
			"ff64ff72", -- Agile
			"ffa6d5ff", -- Evasive
		},
		tStances2x = {
			false,
			false,
			false,
		},
		sStealthedBar = "ffef00ef",
		sStealthedBar2x = false,
		sStealthedBGBar = "ffef00ef",
		sStealthedBorder = "ff000000",
		sStealthedText = "ffffffff",
		sStealthedPattern = "ff000000",
		sStealthedGraphicBorder = "ffffffff",
	},
	bPositionOnChar = false,
	fVerticalPositionOnChar = 1,
	bTransitionColor2x = false,
	nTransitionColorIntensity = 1,
	bAlwaysShow = false,
	nBorderWidth = 0,
	nBarWidth = 320,
	nBarHeight = 32,
	nBarStyle = 3,
	nMasterOpacity = 1,
	nBackgroundBarOpacity = 0,
	nBackgroundOpacity = 0,
	nBorderOpacity = 1,
	nBarHighlightOpacity = 0.6,
	nFadeInTime = 0.1,
	nFadeOutTime = 1,
	nFadeOutDelay = 500,
	bBackgroundColorSameAsBar = true,
	bShowBarFlash = true,
	bOnlyShowWhenReachingFull = false,
	nBarFlashOpacity = 0.7,
	bShowGraphicBorder = true,
	nGraphicBorderID = 1,
	bShowPattern = false,
	nPatternID = 1,
	fPatternOpacity = 0.6,
	nColorMode = 1, -- 1 = constant. // 2 = transition from green to red. // 3 = use stance colors.
	tText = {
		bShow = true,
		nAlign = 3,
		nFont = 3,
		nTextAlign = 3,
		nScale = 0.9,
		nOffsetX = -8,
		nOffsetY = -1,
		fOpacity = 1,
	},
	bUseSmoothUpdate = true,
	bSmoothUpdateText = false,
	tMarkers = {
		bShow = true,
	},
	tStealthed = {
		bBarStyle = true,
		bBarColor = true,
		bBGBarColor = false,
		bBorderColor = false,
		bBorderSize = false,
		bBorderOpacity = false,
		bMasterOpacity = false,
		bHighlightOpacity = true,
		bPattern = true,
		bPatternOpacity = true,
		bPatternColor = true,
		fMasterOpacity = 1,
		fHighlightOpacity = 0.5,
		nBorderWidth = 0,
		fBorderOpacity = 1,
		nBarStyle = 5,
		fTextOpacity = 1,
		nGraphicalBorderID = 1,
		nPatternID = 4,
		fPatternOpacity = 0.8,
	},
	bFirstRun = true,
}

local sSmoothUpdateDisclaimer = "<T Font=\"CRB_Interface9\" TextColor=\"ff9fefff\">%s</T>"
local tSmoothUpdateDisclaimer = {
	"The game only updates your suit power amount at set intervals (twice a second) and because of this the updating of the bar isn't very smooth. To combat it I made this option, but there are some things to take note of.",
	"You can get either 3 or 4 suit power per tick and I have no way of telling which you're going to get, so there is a bit of jerkyness to the smooth updating.",
	"You can choose to update the text as well, however this can be misleading as you don't actually have the value shown between ticks until the next tick hits.",
	"Smooth update is disabled while the options window is open.",
}
local nStealthId = 38784
local nBorderMaxWidth = 8
local nBarStyles = 26
local nBarStylsVertical = 7
local nPatterns = 7
local nBorders = 1
local nTimeBetweenTicks = 500
local nStepsBetweenTicks = 3
local SettingsRef, OptionsWindowRef, ResourceBarRef, nStanceColorIndex
local tColors = {
	tStances = {},
}


function StalkerResourceSystem:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function StalkerResourceSystem:Init()
	Apollo.RegisterAddon(self)
end

function StalkerResourceSystem:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("Options.xml")
	self.xmlDocMain = XmlDoc.CreateFromFile("StalkerResourceSystem.xml")
	self.bCharacterCreated = false
	self.bInCombat = false
	self.bAtMaxResource = false
	self.bOptionsOpen = false
	self.nCurrentStance = 0
	self.nLastValue = 0
	self.nTimePassed = 0
	self.nResourceMax = 100
	self.nLastFrameTime = 0
	self.nFrameCount = 0
	self.nDelayTimePassed = 0
	self.bDelayActive = false
	self.bDoSmoothUpdate = false
	self.bForceBarFlash = false
	self.bForceFlashMouseButtonDown = false
	self.bForceBarFlashColorPickerOpen = false
	self.strCurPosMode = nil
	self.bIsStealthed = false
	self.bMainAnchorShown = false
	self.bForceUpdate = false
	self.bBarStyleStealthed = false
	self.tPatterns = nil

	self.settings = {}
	for k, v in pairs(tDefaultSettings) do
		self.settings[k] = v
	end

	if GameLib.GetPlayerUnit() then
		self:OnCharacterCreated()
	else
		Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
	end
end

function StalkerResourceSystem:OnCharacterCreated()
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer:GetClassId() ~= GameLib.CodeEnumClass.Stalker or self.bCharacterCreated then
		return
	end
	self.bCharacterCreated = true
	Apollo.LoadSprites("VSRS_sprites.xml", "VSRS_sprites")

	Apollo.RegisterSlashCommand("vsrs", "OnSlashCommand", self)

	self:CreateResourceBar()

	self.wndOptions, self.wndBarStyles, self.wndBarStylesList, self.wndFontOptions, self.wndFontOptionsList, self.wndMarkerSystem, self.wndMarkerSystemList, wndPatterns = nil, nil, nil, nil, nil, nil, nil, nil, nil

	Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)
	Apollo.RegisterEventHandler("NextFrame", "OnNextFrame", self)
	Apollo.RegisterEventHandler("ResolutionChanged", "SetPosition", self)	
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("VSRS_ToggleOptions", "ShowHideOptions", self)

	Apollo.RegisterTimerHandler("VSRSStealthWatcher", "OnStealthWatchTimer", self)
	Apollo.RegisterTimerHandler("CreateBarStylesTimer", "OnCreateBarStylesTimer", self)
	Apollo.RegisterTimerHandler("BarFlashColorPickerTimer", "OnBarFlashColorPickerTimer", self)
	Apollo.RegisterTimerHandler("StopBFCPTimer", "OnStopBFCPTimer", self)
	Apollo.RegisterTimerHandler("MainEscapableTimer", "OnMainEscapableTimer", self)
	Apollo.RegisterTimerHandler("VSRSFirstRunTimer", "OnFirstRun", self)

	Apollo.CreateTimer("VSRSStealthWatcher", 0.1, true)
	Apollo.CreateTimer("VSRSFirstRunTimer", 4, false)

	self:SetAllOptions(true)
end

function StalkerResourceSystem:OnFirstRun()
	if self.settings.bFirstRun then
		VILib:WriteToChat("[Stalker Resource System] Type /vsrs to open the options to configure and move the resource bar.")
		VILib:WriteToChat("[Stalker Resource System] You can also access the options through the menu item in the interface menu (the one opened from the bottom left).")
		VILib:WriteToChat("[Stalker Resource System] I hope you find the add-on to your liking. Comments and feedback is much appreciated.")
	end
end

function StalkerResourceSystem:OnStopBFCPTimer()
	Apollo.StopTimer("BarFlashColorPickerTimer")
end

function StalkerResourceSystem:OnConfigure()
	if self.bCharacterCreated then
		self:ShowHideOptions()
	else
		VILib:WriteToChat("Stalker Resource System does not load unless you're playing a Stalker class character.")
	end
end

function StalkerResourceSystem:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Stalker Resource System", {"VSRS_ToggleOptions", "", "VSRS_Sprites:Icon"})
end

function StalkerResourceSystem:CreateResourceBar()
	local strPosMode = ""
	local strFormName = "ResourceContainerForm"
	if self.settings.bPositionOnChar then
		strPosMode = "InWorldHudStratum"
		strFormName = "ResourceCharContainerForm"
	end
	if self.strCurPosMode ~= strPosMode then
		self:DestroyResourceBar()
		self.strCurPosMode = strPosMode
		self.wndResourceContainer = Apollo.LoadForm(self.xmlDocMain, strFormName, strPosMode, self)
		self.wndResourceBar = Apollo.LoadForm(self.xmlDocMain, "ResourceForm", self.wndResourceContainer, self)
		if self.settings.bShowGraphicBorder then
			self.wndResourceBarBorder = Apollo.LoadForm(self.xmlDocMain, "GraphicBorderForm", self.wndResourceContainer, self)
		end
		self.wndResourceBar:FindChild("Background"):SetSprite("VSRS_sprites:Background")
		self.wndResourceBar:FindChild("BarFlash"):SetSprite("VSRS_sprites:BarFlash")
		self.wndResourceBar:FindChild("BorderLeft"):SetSprite("VSRS_sprites:Border_LR")
		self.wndResourceBar:FindChild("BorderTop"):SetSprite("VSRS_sprites:Border_TB")
		self.wndResourceBar:FindChild("BorderRight"):SetSprite("VSRS_sprites:Border_LR")
		self.wndResourceBar:FindChild("BorderBottom"):SetSprite("VSRS_sprites:Border_TB")
		self.wndResourceBar:FindChild("BarFlash"):Show(false, true)
		self.wndResourceContainer:ToFront()
		if not self.bOptionsOpen and not self.settings.bAlwaysShow then self.wndResourceContainer:Show(false, true) end
		self:SetAllOptions(false, true)
		self.wndResourceContainer:SetStyle("Moveable", self.bOptionsOpen)
		self.wndResourceContainer:SetStyle("IgnoreMouse", not self.bOptionsOpen)
	end
	local fScale = 1 / Apollo.GetConsoleVariable("ui.Scale")
	self.wndResourceContainer:SetScale(fScale)
end

function StalkerResourceSystem:DestroyResourceBar()
	if self.wndResourceContainer then
		self.wndResourceBar:Destroy()
		self.wndResourceBar = nil
		if self.wndResourceBarBorder then
			self.wndResourceBarBorder:Destroy()
			self.wndResourceBarBorder = nil	
		end
		self.wndResourceContainer:Destroy()
		self.wndResourceContainer = nil
	end
end

function StalkerResourceSystem:SetAllOptions(bSkipBar, bSkipColors)
	if not bSkipColors then
		tColors.sText = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sText))
		tColors.sBarTest = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sBarTest))
		tColors.sBar = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sBar))
		tColors.sBarFlash = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sBarFlash))
		tColors.sBackground = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sBackground))
		tColors.sBackgroundBar = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sBackgroundBar))
		tColors.sBorder = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sBorder))
		tColors.sPattern = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sPattern))
		tColors.sGraphicBorder = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sGraphicBorder))
		for i = 1, 3 do
			tColors.tStances[i] = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.tStances[i]))
		end
		tColors.sStealthedBar = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sStealthedBar))
		tColors.sStealthedBGBar = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sStealthedBGBar))
		tColors.sStealthedBorder = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sStealthedBorder))
		tColors.sStealthedText = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sStealthedText))
		tColors.sStealthedPattern = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sStealthedPattern))
		tColors.sStealthedGraphicBorder = CColor.new(VILib:ConvertHexToColor(self.settings.tColors.sStealthedGraphicBorder))
	end
	
	if not bSkipBar then
		self:SetPosition()
		self:SetBorderSettings()
		self:SetFontSettings()
		self:SetBarStyle()
		self:SetBarHighlight()
		self:SetBarFlashOptions()
		self:SetPatternOverlayOptions()
		self:SetGraphicBorder()
		self:SetGraphicBorderColor()
	end
end

function StalkerResourceSystem:OnNextFrame()
	local nTickCount = Apollo.GetTickCount()
	local nTimePassed = nTickCount - self.nLastFrameTime
	self.nFrameCount = self.nFrameCount + 1

	if self.bDelayActive then
		self.nDelayTimePassed = self.nDelayTimePassed + nTimePassed
		if self.bInCombat or self.bOptionsOpen then
			self.bDelayActive = false
			self.nDelayTimePassed = 0
		elseif self.nDelayTimePassed >= self.settings.nFadeOutDelay then
			self.bDelayActive = false
			self.nDelayTimePassed = 0
			self.wndResourceContainer:Show(false, false, self.settings.nFadeOutTime)
		end
	end

	if self.settings.bUseSmoothUpdate and not self.bOptionsOpen then
		self.nTimePassed = self.nTimePassed + nTimePassed
		if self.bDoSmoothUpdate and self.nFrameCount < 4 then
			self:UpdateBar(self.nLastValue + (nStepsBetweenTicks * (self.nTimePassed / nTimeBetweenTicks)), true)
		end
	end

	self.nLastFrameTime = nTickCount

	if self.nFrameCount >= 4 or self.bForceUpdate then
		self.bForceUpdate = false
		local unitPlayer = GameLib.GetPlayerUnit()
		local nResource3 = unitPlayer:GetResource(3)
		self.nResourceMax = unitPlayer:GetMaxResource(3)

		local nStanceIdx = GameLib.GetCurrentClassInnateAbilityIndex() or 1
	    if self.nCurrentStance ~= nStanceIdx then
			self.nCurrentStance = nStanceIdx
			self:UpdateBar(nResource3, false)
		end

		if nResource3 == self.nResourceMax then
			self.bDoSmoothUpdate = false
			self.nTimePassed = 0
			self.bAtMaxResource = true
			if not self.bInCombat and not self.bOptionsOpen then
				self.wndResourceBar:FindChild("MarkerContainer"):Show(false, false, 0.5)
				if not self.settings.bAlwaysShow then self.bDelayActive = true end
			elseif self.bInCombat then
				if self.settings.bShowBarFlash then
					local bDisplayFlash = true
					if self.settings.bOnlyShowWhenReachingFull and self.nLastValue == self.nResourceMax then
						bDisplayFlash = false
					end
					if bDisplayFlash then self.wndResourceBar:FindChild("BarFlash"):Show(true, true) end
				end
			end
			if self.bForceBarFlash then
				self.wndResourceBar:FindChild("BarFlash"):Show(true, true)
			end
		else
			self.bDelayActive = false
			self.bDoSmoothUpdate = true
			self.wndResourceBar:FindChild("BarFlash"):Show(false, false, 0.4)
			if not self.wndResourceContainer:IsVisible() then
				self.wndResourceContainer:Show(true, false, self.settings.nFadeInTime)
			end
		end

		if self.nLastValue ~= nResource3 then
			if self.nLastValue < nResource3 then self.nTimePassed = 0 end
			self.nLastValue = nResource3
			self:UpdateBar(nResource3, false)
		end
		if self.bOptionsOpen then self:UpdateBar(nResource3, false) end

		self.nFrameCount = 0
	end
end

function StalkerResourceSystem:UpdateBar(nResource, bFromSmooth)
	self.wndResourceBar:FindChild("ProgressBar"):SetMax(self.nResourceMax)
	self.wndResourceBar:FindChild("ProgressBar"):SetProgress(nResource)
	self.wndResourceBar:FindChild("ProgressBarPattern"):SetMax(self.nResourceMax)
	self.wndResourceBar:FindChild("ProgressBarPattern"):SetProgress(nResource)
	if not bFromSmooth or (bFromSmooth and self.settings.bSmoothUpdateText) then
		self.wndResourceBar:FindChild("ProgressBarText"):SetText(VILib:Round(nResource))
	end
	if self.settings.nBarHighlightOpacity > 0 then
		self.wndResourceBar:FindChild("ProgressBarHighlight"):SetMax(self.nResourceMax)
		self.wndResourceBar:FindChild("ProgressBarHighlight"):SetProgress(nResource)
	end

	local sColor
	if self.bIsStealthed and self.settings.tStealthed.bBarColor then
		sColor = self:GetColor("StealthedBar")
	else
		if self.settings.nColorMode == 2 then
			local nPercent = nResource / self.nResourceMax
			if nPercent > 1 then nPercent = 1 end
			local r, g = 1, 1
			if nPercent > 0.5 then
				r = (r - nPercent) * 2
			else
				g = nPercent * 2
			end
			local nVal = 255 * self.settings.nTransitionColorIntensity
			sColor = string.format("%02x%02x%02x%02x", 0xFF, VILib:Round(nVal * r), VILib:Round(nVal * g), 0x00)
			if self.settings.bTransitionColor2x then sColor = "2x:" .. sColor end
		elseif self.settings.nColorMode == 3 then
			sColor = self:GetColor("Stances", self.nCurrentStance)
		else
			sColor = self:GetColor("Bar")
		end
	end

	self.wndResourceBar:FindChild("ProgressBar"):SetBarColor(sColor)
	if self.settings.bBackgroundColorSameAsBar then
		self.wndResourceBar:FindChild("ProgressBarBG"):SetBGColor(sColor)
	else
		if self.bIsStealthed and self.settings.tStealthed.bBGBarColor then
			self.wndResourceBar:FindChild("ProgressBarBG"):SetBGColor(self:GetColor("StealthedBGBar"))
		else
			self.wndResourceBar:FindChild("ProgressBarBG"):SetBGColor(self:GetColor("BackgroundBar"))
		end
	end
end

function StalkerResourceSystem:OnStealthWatchTimer()
	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then return end
	local buffs = unitPlayer:GetBuffs()
	if buffs then
		local bIsStealthed = false
		for k, v in pairs(buffs.arBeneficial) do
			if v.splEffect:GetId() == nStealthId then
				if not self.bIsStealthed then
					self.bIsStealthed = true
					self:UpdateStealth()
				end
				bIsStealthed = true
				break
			end
		end
		if not bIsStealthed and self.bIsStealthed then
			self.bIsStealthed = false
			self:UpdateStealth()
		end
	end
end

function StalkerResourceSystem:UpdateStealth()
	if self.bIsStealthed then
		if self.settings.tStealthed.bBarStyle then self:SetStealthedBarStyle() end
		if self.settings.tStealthed.bBarColor then self.wndResourceBar:FindChild("ProgressBar"):SetBarColor(self:GetColor("StealthedBar")) end
		if self.settings.tStealthed.bBGBarColor and not self.settings.bBackgroundColorSameAsBar then self.wndResourceBar:FindChild("ProgressBarBG"):SetBBGColor(self:GetColor("StealthedBGBar")) end
		if self.settings.tStealthed.bMasterOpacity then self.wndResourceContainer:SetOpacity(self.settings.tStealthed.fMasterOpacity, 100) end
		if self.settings.tStealthed.bHighlightOpacity then self.wndResourceBar:FindChild("ProgressBarHighlight"):SetOpacity(self.settings.tStealthed.fHighlightOpacity, 100) end
		if self.settings.tStealthed.bTextColor then self.wndResourceBar:FindChild("ProgressBarText"):SetTextColor(self.settings.tColors.sStealthedText) end
		if self.settings.tStealthed.bTextOpacity then self.wndResourceBar:FindChild("ProgressBarText"):SetOpacity(self.settings.tStealthed.fTextOpacity, 100) end
		self.wndResourceBar:FindChild("ProgressBarPattern"):SetFullSprite("VSRS_sprites:Pattern" .. self.settings.tStealthed.nPatternID)
		if self.settings.tStealthed.bPattern then self.wndResourceBar:FindChild("ProgressBarPattern"):Show(true, true) end
		if self.settings.tStealthed.bPatternOpacity then self.wndResourceBar:FindChild("ProgressBarPattern"):SetOpacity(self.settings.tStealthed.fPatternOpacity, 100) end
		if self.settings.tStealthed.bPatternColor then self.wndResourceBar:FindChild("ProgressBarPattern"):SetBarColor(self:GetColor("StealthedPattern")) end
		
		self:SetStealthedBorderSettings()
	else
		if self.settings.tStealthed.bMasterOpacity then self.wndResourceContainer:SetOpacity(self.settings.nMasterOpacity, 100) end
		if self.settings.tStealthed.bHighlightOpacity then self.wndResourceBar:FindChild("ProgressBarHighlight"):SetOpacity(self.settings.nBarHighlightOpacity, 100) end
		if self.settings.tStealthed.bTextColor then self.wndResourceBar:FindChild("ProgressBarText"):SetTextColor(self.settings.tColors.sText) end
		if self.settings.tStealthed.bTextOpacity then self.wndResourceBar:FindChild("ProgressBarText"):SetOpacity(self.settings.tText.fOpacity, 100) end
		self.wndResourceBar:FindChild("ProgressBarPattern"):SetFullSprite("VSRS_sprites:Pattern" .. self.settings.nPatternID)
		self:SetBorderSettings()
		self:SetBarStyle()
		self:SetBarHighlight()
		self:SetPatternOverlayOptions()
	end
	self.bForceUpdate = true
end

function StalkerResourceSystem:SetStealthedBarStyle()
	self.wndResourceBar:FindChild("ProgressBar"):SetFullSprite("VSRS_sprites:ProgressBarH" .. self.settings.tStealthed.nBarStyle)
	self.wndResourceBar:FindChild("ProgressBarBG"):SetSprite("VSRS_sprites:ProgressBarH" .. self.settings.tStealthed.nBarStyle)
	if not tNoHighlight[self.settings.tStealthed.nBarStyle] and self.settings.tStealthed.fHighlightOpacity > 0 then
		self.wndResourceBar:FindChild("ProgressBarHighlight"):SetFullSprite("VSRS_sprites:ProgressBarH" .. self.settings.tStealthed.nBarStyle .. "_Hi")
		self.wndResourceBar:FindChild("ProgressBarHighlight"):Show(true, true)
	else
		self.wndResourceBar:FindChild("ProgressBarHighlight"):Show(false, true)
	end
end

function StalkerResourceSystem:GetColor(key, index)
	local sColor = "ffffffff"
	local sPrefix = "s"
	if index then sPrefix = "t" end
	if self.settings.tColors[sPrefix .. key] then
		if sPrefix == "t" then
			sColor = self.settings.tColors[sPrefix .. key][index]
			if self.settings.tColors["t" .. key .. "2x"] then
				if self.settings.tColors["t" .. key .. "2x"][index] then
					sColor = "2x:" .. sColor
				end
			end
		else
			sColor = self.settings.tColors[sPrefix .. key]
			if self.settings.tColors["b" .. key .. "2x"] then
				sColor = "2x:" .. sColor
			end
		end
	end
	return sColor
end

function StalkerResourceSystem:SetPosition()
	local nWidth, nHeight = self.settings.nBarWidth + (nBorderMaxWidth * 2), self.settings.nBarHeight + (nBorderMaxWidth * 2)
	local fScale = 1 - (1 / Apollo.GetConsoleVariable("ui.Scale"))
	local nOffsetX, nOffsetY = VILib:Round(math.abs(nWidth * fScale) / 2), VILib:Round(math.abs(nHeight * fScale) / 2)

	if self.settings.bPositionOnChar then
		local nX, nY = unpack(self.settings.tWindowPositions.tMainCharOffsets)
		if tonumber(nX) and tonumber(nY) then
			local nL, nT, nR, nB = VILib:Round(nWidth * -0.5) + nX, VILib:Round(nHeight * -0.5) + nY, VILib:Round(nWidth * 0.5) + nX, VILib:Round(nHeight * 0.5) + nY
			self.wndResourceContainer:SetAnchorOffsets(nL - nOffsetX, nT - nOffsetY, nR - nOffsetX, nB - nOffsetY)
			local unitPlayer = GameLib.GetPlayerUnit()
			if unitPlayer then self.wndResourceContainer:SetUnit(unitPlayer, self.settings.fVerticalPositionOnChar) end
		end
	else
		self.wndResourceContainer:SetAnchorPoints(unpack(tAnchors[self.settings.tWindowPositions.nMainAnchor].tPoints))
		local nX, nY = unpack(self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor])
		if tonumber(nX) and tonumber(nY) then
			local nOL, nOT, nOR, nOB = unpack(tAnchors[self.settings.tWindowPositions.nMainAnchor].tOffsetMults)
			local nL, nT, nR, nB = VILib:Round(nWidth * nOL) + nX, VILib:Round(nHeight * nOT) + nY, VILib:Round(nWidth * nOR) + nX, VILib:Round(nHeight * nOB) + nY
			self.wndResourceContainer:SetAnchorOffsets(nL - nOffsetX, nT - nOffsetY, nR - nOffsetX, nB - nOffsetY)
		end
	end
	self.wndResourceBar:SetAnchorOffsets(tBorderOffsets[self.settings.nGraphicBorderID][1] - nBorderMaxWidth, tBorderOffsets[self.settings.nGraphicBorderID][2] - nBorderMaxWidth, tBorderOffsets[self.settings.nGraphicBorderID][3] + nBorderMaxWidth, tBorderOffsets[self.settings.nGraphicBorderID][4] + nBorderMaxWidth)
end

function StalkerResourceSystem:SetBarSize()
	self:SetPosition()
end

function StalkerResourceSystem:SetBorderSettings()
	local nBorderWidthDiff = nBorderMaxWidth - self.settings.nBorderWidth
	self.wndResourceBar:FindChild("BorderLeft"):SetAnchorOffsets(nBorderWidthDiff, nBorderWidthDiff, nBorderMaxWidth, -nBorderWidthDiff)
	self.wndResourceBar:FindChild("BorderLeft"):SetBGColor(self.settings.tColors.sBorder)
	self.wndResourceBar:FindChild("BorderLeft"):SetOpacity(self.settings.nBorderOpacity, 100)
	self.wndResourceBar:FindChild("BorderTop"):SetAnchorOffsets(nBorderMaxWidth, nBorderWidthDiff, -nBorderMaxWidth, nBorderMaxWidth)
	self.wndResourceBar:FindChild("BorderTop"):SetBGColor(self.settings.tColors.sBorder)
	self.wndResourceBar:FindChild("BorderTop"):SetOpacity(self.settings.nBorderOpacity, 100)
	self.wndResourceBar:FindChild("BorderRight"):SetAnchorOffsets(-nBorderMaxWidth, nBorderWidthDiff, -nBorderWidthDiff, -nBorderWidthDiff)
	self.wndResourceBar:FindChild("BorderRight"):SetBGColor(self.settings.tColors.sBorder)
	self.wndResourceBar:FindChild("BorderRight"):SetOpacity(self.settings.nBorderOpacity, 100)
	self.wndResourceBar:FindChild("BorderBottom"):SetAnchorOffsets(nBorderMaxWidth, -nBorderMaxWidth, -nBorderMaxWidth, -nBorderWidthDiff)
	self.wndResourceBar:FindChild("BorderBottom"):SetBGColor(self.settings.tColors.sBorder)
	self.wndResourceBar:FindChild("BorderBottom"):SetOpacity(self.settings.nBorderOpacity, 100)
end

function StalkerResourceSystem:SetStealthedBorderSettings()
	if self.settings.tStealthed.bBorderSize then
		local nBorderWidthDiff = nBorderMaxWidth - self.settings.tStealthed.nBorderWidth
		self.wndResourceBar:FindChild("BorderLeft"):SetAnchorOffsets(nBorderWidthDiff, nBorderWidthDiff, nBorderMaxWidth, -nBorderWidthDiff)
		self.wndResourceBar:FindChild("BorderTop"):SetAnchorOffsets(nBorderMaxWidth, nBorderWidthDiff, -nBorderMaxWidth, nBorderMaxWidth)
		self.wndResourceBar:FindChild("BorderRight"):SetAnchorOffsets(-nBorderMaxWidth, nBorderWidthDiff, -nBorderWidthDiff, -nBorderWidthDiff)
		self.wndResourceBar:FindChild("BorderBottom"):SetAnchorOffsets(nBorderMaxWidth, -nBorderMaxWidth, -nBorderMaxWidth, -nBorderWidthDiff)
	end
	if self.settings.tStealthed.bBorderColor then
		self.wndResourceBar:FindChild("BorderLeft"):SetBGColor(self.settings.tColors.sStealthedBorder)
		self.wndResourceBar:FindChild("BorderTop"):SetBGColor(self.settings.tColors.sStealthedBorder)
		self.wndResourceBar:FindChild("BorderRight"):SetBGColor(self.settings.tColors.sStealthedBorder)
		self.wndResourceBar:FindChild("BorderBottom"):SetBGColor(self.settings.tColors.sStealthedBorder)
	end
	if self.settings.tStealthed.bBorderOpacity then
		self.wndResourceBar:FindChild("BorderLeft"):SetOpacity(self.settings.tStealthed.fBorderOpacity, 100)
		self.wndResourceBar:FindChild("BorderTop"):SetOpacity(self.settings.tStealthed.fBorderOpacity, 100)
		self.wndResourceBar:FindChild("BorderRight"):SetOpacity(self.settings.tStealthed.fBorderOpacity, 100)
		self.wndResourceBar:FindChild("BorderBottom"):SetOpacity(self.settings.tStealthed.fBorderOpacity, 100)
	end
end

function StalkerResourceSystem:SetFontSettings()
	self.wndResourceBar:FindChild("ProgressBarText"):Show(self.settings.tText.bShow, true)
	self.wndResourceBar:FindChild("ProgressBarText"):SetOpacity(self.settings.tText.fOpacity, 1000)
	self.wndResourceBar:FindChild("ProgressBarText"):SetFont(tFonts[self.settings.tText.nFont][1])
	self.wndResourceBar:FindChild("ProgressBarText"):SetTextFlags("DT_CENTER", false)
	self.wndResourceBar:FindChild("ProgressBarText"):SetTextFlags("DT_RIGHT", false)
	if self.settings.tText.nTextAlign > 1 then
		self.wndResourceBar:FindChild("ProgressBarText"):SetTextFlags("DT_" .. arTextAligns[self.settings.tText.nTextAlign]:upper(), true)
	end
	self.wndResourceBar:FindChild("ProgressBarText"):SetTextColor(self.settings.tColors.sText)
	self.wndResourceBar:FindChild("ProgressBarText"):SetScale(self.settings.tText.nScale * Apollo.GetConsoleVariable("ui.Scale"))
	local fScale = 1 - (1 / Apollo.GetConsoleVariable("ui.Scale"))
	local nWidth, nHeight = 40, 20
	local nYOffset = ((nHeight * self.settings.tText.nScale) - nHeight) + self.settings.tText.nOffsetY - VILib:Round(math.abs((nHeight * fScale) / 2))
	if self.settings.tText.nAlign == 1 then
		local nXOffset = ((nWidth * 2) * self.settings.tText.nScale) - (nWidth * 2) - VILib:Round(math.abs(nWidth * fScale))
		self.wndResourceBar:FindChild("ProgressBarText"):SetAnchorPoints(0, 0.5, 0, 0.5)
		self.wndResourceBar:FindChild("ProgressBarText"):SetAnchorOffsets(0 + self.settings.tText.nOffsetX, -nHeight - nYOffset, (nWidth * 2) + self.settings.tText.nOffsetX, nHeight - nYOffset)
	elseif self.settings.tText.nAlign == 3 then
		local nXOffset = ((nWidth * 2) * self.settings.tText.nScale) - (nWidth * 2) - VILib:Round(math.abs(nWidth * fScale))
		self.wndResourceBar:FindChild("ProgressBarText"):SetAnchorPoints(1, 0.5, 1, 0.5)
		self.wndResourceBar:FindChild("ProgressBarText"):SetAnchorOffsets(-(nWidth * 2) - nXOffset + self.settings.tText.nOffsetX, -nHeight - nYOffset, 0 - nXOffset + self.settings.tText.nOffsetX, nHeight - nYOffset)
	else
		local nXOffset = ((nWidth * self.settings.tText.nScale) - nWidth) + self.settings.tText.nOffsetX - VILib:Round(math.abs(nWidth * fScale))
		self.wndResourceBar:FindChild("ProgressBarText"):SetAnchorPoints(0.5, 0.5, 0.5, 0.5)
		self.wndResourceBar:FindChild("ProgressBarText"):SetAnchorOffsets(-nWidth - nXOffset, -nHeight - nYOffset, nWidth - nXOffset, nHeight - nYOffset)
	end
end

function StalkerResourceSystem:SetBarStyle()
	self.wndResourceBar:FindChild("ProgressBar"):SetFullSprite("VSRS_sprites:ProgressBarH" .. self.settings.nBarStyle)
	self.wndResourceBar:FindChild("ProgressBarBG"):SetSprite("VSRS_sprites:ProgressBarH" .. self.settings.nBarStyle)
	self.wndResourceBar:FindChild("ProgressBarBG"):SetOpacity(self.settings.nBackgroundBarOpacity, 100)
	self.wndResourceBar:FindChild("Background"):SetOpacity(self.settings.nBackgroundOpacity, 100)
	self.wndResourceBar:FindChild("Background"):SetBGColor(self.settings.tColors.sBackground)
	self.wndResourceBar:FindChild("BGFlash"):SetSprite("VSRS_sprites:ProgressBarH" .. self.settings.nBarStyle)
	self.wndResourceBar:FindChild("BGFlash"):SetBGColor(self:GetColor("BGFlash1"))
	self.wndResourceBar:FindChild("BGFlash"):Show(false, true)
	self.wndResourceContainer:SetOpacity(self.settings.nMasterOpacity, 100)
end

function StalkerResourceSystem:SetBarHighlight()
	if not tNoHighlight[self.settings.nBarStyle] and self.settings.nBarHighlightOpacity > 0 then
		self.wndResourceBar:FindChild("ProgressBarHighlight"):SetFullSprite("VSRS_sprites:ProgressBarH" .. self.settings.nBarStyle .. "_Hi")
		self.wndResourceBar:FindChild("ProgressBarHighlight"):Show(true, true)
		self:SetBarHighlightOpacity()
	else
		self.wndResourceBar:FindChild("ProgressBarHighlight"):Show(false, true)
	end
end

function StalkerResourceSystem:SetBarHighlightOpacity()
	self.wndResourceBar:FindChild("ProgressBarHighlight"):SetOpacity(self.settings.nBarHighlightOpacity, 100)
end

function StalkerResourceSystem:SetBarFlashOptions()
	self.wndResourceBar:FindChild("BarFlash"):SetOpacity(self.settings.nBarFlashOpacity, 100)
end

function StalkerResourceSystem:SetPatternOverlayOptions()
	self.wndResourceBar:FindChild("ProgressBarPattern"):Show(self.settings.bShowPattern, true)
	self.wndResourceBar:FindChild("ProgressBarPattern"):SetFullSprite("VSRS_sprites:Pattern" .. self.settings.nPatternID)
	self.wndResourceBar:FindChild("ProgressBarPattern"):SetOpacity(self.settings.fPatternOpacity, 100)
	self.wndResourceBar:FindChild("ProgressBarPattern"):SetBarColor(self:GetColor("Pattern"))
end

function StalkerResourceSystem:SetGraphicBorder()
	if self.settings.bShowGraphicBorder then
		if not self.wndResourceBarBorder then
			self.wndResourceBarBorder = Apollo.LoadForm(self.xmlDocMain, "GraphicBorderForm", self.wndResourceContainer, self)
		end
		self.wndResourceBarBorder:Show(true, true)
	else
		if self.wndResourceBarBorder then
			self.wndResourceBarBorder:Destroy()
			self.wndResourceBarBorder = nil
		end
	end
end

function StalkerResourceSystem:SetGraphicBorderColor()
	if self.wndResourceBarBorder then
		self.wndResourceBarBorder:SetBGColor(self:GetColor("GraphicBorder"))
	end
end

function StalkerResourceSystem:OnEnteredCombat(unit, bInCombat)
	if unit == GameLib.GetPlayerUnit() then
		if bInCombat == true then
			self.wndResourceContainer:Show(true, false, self.settings.nFadeInTime)
		elseif bInCombat == false and self.bAtMaxResource then
			if self.settings.bAlwaysShow then self.wndResourceBar:FindChild("BarFlash"):Show(false, false, 0.5) end
			if not self.settings.bAlwaysShow then self.wndResourceContainer:Show(false, false, self.settings.nFadeOutTime) end
		end
		self.bInCombat = bInCombat
	end
end

function StalkerResourceSystem:OnSlashCommand(strCmd, strParam)
	if strParam ~= "" then
		if string.lower(strParam) == "reset" then
			for k, v in pairs(self.settings.tWindowPositions) do
				self.settings.tWindowPositions[k] = {}
			end
			self:SetPosition()
			VILib:WriteToChat("Resource bar and all window positions has been reset.")
		end
	else
		self:ShowHideOptions()
	end
end

function StalkerResourceSystem:ShowHideOptions(onlyBar)
	self.bOptionsOpen = not self.bOptionsOpen
	if not onlyBar then
		if not self.wndOptions then
			self.wndOptions = Apollo.LoadForm(self.xmlDoc, "OptionsForm", nil, self)
			self.wndOptionsSizePosition = Apollo.LoadForm(self.xmlDoc, "OptionsSizePositioning", self.wndOptions:FindChild("ContentFrame"), self)
			self.wndOptionsBarLook = Apollo.LoadForm(self.xmlDoc, "OptionsBarLook", self.wndOptions:FindChild("ContentFrame"), self)
			self.wndOptionsBarPattern = Apollo.LoadForm(self.xmlDoc, "OptionsBarPattern", self.wndOptions:FindChild("ContentFrame"), self)
			self.wndOptionsBarBorder = Apollo.LoadForm(self.xmlDoc, "OptionsBarBorder", self.wndOptions:FindChild("ContentFrame"), self)
			self.wndOptionsBarFlash = Apollo.LoadForm(self.xmlDoc, "OptionsBarFlash", self.wndOptions:FindChild("ContentFrame"), self)
			self.wndOptionsText = Apollo.LoadForm(self.xmlDoc, "OptionsText", self.wndOptions:FindChild("ContentFrame"), self)
			self.wndOptionsStealthed = Apollo.LoadForm(self.xmlDoc, "OptionsStealthed", self.wndOptions:FindChild("ContentFrame"), self)
			self.wndOptionsSpecials = Apollo.LoadForm(self.xmlDoc, "OptionsSpecials", self.wndOptions:FindChild("ContentFrame"), self)
			self.wndOptionsAdvanced = Apollo.LoadForm(self.xmlDoc, "OptionsAdvanced", self.wndOptions:FindChild("ContentFrame"), self)
			self:OnOptionsMenuButtonCheck()
			self.wndOptions:FindChild("Menu:VersionText"):SetText("Version " .. sVersion)
		end
		if self.bOptionsOpen then
			self:SetOptions()
			self.wndOptions:ToFront()
			local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
			local nLeft, nTop, nRight, nBottom = self.wndOptions:GetRect()
			local nWidth, nHeight = nRight - nLeft, nBottom - nTop
			if self.settings.tWindowPositions.tOptions[1] then
				self.wndOptions:Move(VILib:Round(nScreenWidth * self.settings.tWindowPositions.tOptions[1]), VILib:Round(nScreenHeight * self.settings.tWindowPositions.tOptions[2]), nWidth, nHeight)
			else
				self.wndOptions:Move(VILib:Round((nScreenWidth / 2) - (nWidth / 2)), VILib:Round((nScreenHeight / 2) - (nHeight / 2)), nWidth, nHeight)
			end
			OptionsWindowRef = self.wndOptions
			SettingsRef = self.settings
			ResourceBarRef = self.wndResourceBar
		else
			if self.wndBarStyles then self:ShowHideBarStyleOptions() end
			if self.wndFontOptions then self:ShowHideFontOptions() end
			if self.wndMarkerSystem then self:ShowHideMarkerSystem() end
		end
		self.wndOptions:Show(self.bOptionsOpen, false)
	end
	if not self.bOptionsOpen and not self.bInCombat and not self.settings.bAlwaysShow then
		self.wndResourceContainer:Show(self.bOptionsOpen, self.bOptionsOpen)
	elseif self.bOptionsOpen and not self.wndResourceContainer:IsVisible() then
		self.wndResourceContainer:Show(self.bOptionsOpen, self.bOptionsOpen)
	end
	self.wndResourceContainer:SetStyle("Moveable", self.bOptionsOpen)
	self.wndResourceContainer:SetStyle("IgnoreMouse", not self.bOptionsOpen)
end

function StalkerResourceSystem:SetOptions()
	self.wndOptionsSizePosition:FindChild("BarWidth"):FindChild("Slider"):SetValue(self.settings.nBarWidth)
	self.wndOptionsSizePosition:FindChild("BarWidth"):FindChild("Output"):SetText(self.settings.nBarWidth)
	self.wndOptionsSizePosition:FindChild("BarHeight"):FindChild("Slider"):SetValue(self.settings.nBarHeight)
	self.wndOptionsSizePosition:FindChild("BarHeight"):FindChild("Output"):SetText(self.settings.nBarHeight)
	self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetX:Slider"):SetValue(self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][1])
	self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetX:Output"):SetText(self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][1])
	self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetY:Slider"):SetValue(self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][2])
	self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetY:Output"):SetText(self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][2])
	self.wndOptionsSizePosition:FindChild("CharacterPosContainer:TiePositionToCharacter:Button"):SetCheck(self.settings.bPositionOnChar)
	self.wndOptionsSizePosition:FindChild("CharacterPosContainer:VerticalPositionOnCharacter:Slider"):SetValue(self.settings.fVerticalPositionOnChar)
	self.wndOptionsSizePosition:FindChild("CharacterPosContainer:VerticalPositionOnCharacter:Output"):SetText(arPositionOnCharacterNames[self.settings.fVerticalPositionOnChar + 1])
	self.wndOptionsSizePosition:FindChild("CharacterPosContainer:PosOffsetX:Slider"):SetValue(self.settings.tWindowPositions.tMainCharOffsets[1])
	self.wndOptionsSizePosition:FindChild("CharacterPosContainer:PosOffsetX:Output"):SetText(self.settings.tWindowPositions.tMainCharOffsets[1])
	self.wndOptionsSizePosition:FindChild("CharacterPosContainer:PosOffsetY:Slider"):SetValue(self.settings.tWindowPositions.tMainCharOffsets[2])
	self.wndOptionsSizePosition:FindChild("CharacterPosContainer:PosOffsetY:Output"):SetText(self.settings.tWindowPositions.tMainCharOffsets[2])
	self:FillAnchorList()

	self.wndOptionsBarLook:FindChild("AlwaysShow"):FindChild("Button"):SetCheck(self.settings.bAlwaysShow)
	self.wndOptionsBarLook:FindChild("BarStylesButton"):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.sBarTest)
	self.wndOptionsBarLook:FindChild("BarStylesButton"):FindChild("2xButton"):SetCheck(self.settings.tColors.bBarTest2x)
	self.wndOptionsBarLook:FindChild("MasterOpacity"):FindChild("Slider"):SetValue(VILib:Round(self.settings.nMasterOpacity * 100))
	self.wndOptionsBarLook:FindChild("MasterOpacity"):FindChild("Output"):SetText(VILib:Round(self.settings.nMasterOpacity * 100) .. "%")
	self.wndOptionsBarLook:FindChild("BackgroundBarOpacity"):FindChild("Slider"):SetValue(VILib:Round(self.settings.nBackgroundBarOpacity * 100))
	self.wndOptionsBarLook:FindChild("BackgroundBarOpacity"):FindChild("Output"):SetText(VILib:Round(self.settings.nBackgroundBarOpacity * 100) .. "%")
	self.wndOptionsBarLook:FindChild("BackgroundOpacity"):FindChild("Slider"):SetValue(VILib:Round(self.settings.nBackgroundOpacity * 100))
	self.wndOptionsBarLook:FindChild("BackgroundOpacity"):FindChild("Output"):SetText(VILib:Round(self.settings.nBackgroundOpacity * 100) .. "%")
	self.wndOptionsBarLook:FindChild("BackgroundColor"):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.sBackground)
	self.wndOptionsBarLook:FindChild("HighlightOpacity"):FindChild("Slider"):SetValue(VILib:Round(self.settings.nBarHighlightOpacity * 100))
	self.wndOptionsBarLook:FindChild("HighlightOpacity"):FindChild("Output"):SetText(VILib:Round(self.settings.nBarHighlightOpacity * 100) .. "%")
	self.wndOptionsBarLook:FindChild("ColorMode" .. self.settings.nColorMode):FindChild("Checkbox"):SetCheck(true)
	self.wndOptionsBarLook:FindChild("ColorMode1Constant"):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.sBar)
	self.wndOptionsBarLook:FindChild("ColorMode1Constant"):FindChild("2xButton"):SetCheck(self.settings.tColors.bBar2x)
	self.wndOptionsBarLook:FindChild("ColorMode2Intensity"):FindChild("2xButton"):SetCheck(self.settings.bTransitionColor2x)
	self.wndOptionsBarLook:FindChild("ColorMode2Intensity"):FindChild("Slider"):SetValue(VILib:Round(self.settings.nTransitionColorIntensity * 100))
	self.wndOptionsBarLook:FindChild("ColorMode2Intensity"):FindChild("Output"):SetText(VILib:Round(self.settings.nTransitionColorIntensity * 100) .. "%")
	for i = 1, 3 do
		self.wndOptionsBarLook:FindChild("ColorMode" .. i):FindChild("Checkbox"):SetData(i)
		self.wndOptionsBarLook:FindChild("ColorMode3Stance" .. i):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.tStances[i])
		self.wndOptionsBarLook:FindChild("ColorMode3Stance" .. i):FindChild("2xButton"):SetCheck(self.settings.tColors.tStances2x[i])
		self.wndOptionsBarLook:FindChild("ColorMode3Stance" .. i):SetData(i)
	end
	self.wndOptionsBarLook:FindChild("BackgroundBarSameAsMain"):FindChild("Button"):SetCheck(self.settings.bBackgroundColorSameAsBar)
	self.wndOptionsBarLook:FindChild("BackgroundBarColor"):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.sBackgroundBar)
	self.wndOptionsBarLook:FindChild("BackgroundBarColor"):FindChild("2xButton"):SetCheck(self.settings.tColors.bBackgroundBar2x)

	self.wndOptionsBarPattern:FindChild("PatternShow"):FindChild("Button"):SetCheck(self.settings.bShowPattern)
	self.wndOptionsBarPattern:FindChild("PatternOpacity"):FindChild("Slider"):SetValue(VILib:Round(self.settings.fPatternOpacity * 100))
	self.wndOptionsBarPattern:FindChild("PatternOpacity"):FindChild("Output"):SetText(VILib:Round(self.settings.fPatternOpacity * 100) .. "%")
	self.wndOptionsBarPattern:FindChild("PatternColor"):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.sPattern)

	self.wndOptionsBarBorder:FindChild("BorderWidth"):FindChild("Slider"):SetValue(self.settings.nBorderWidth)
	self.wndOptionsBarBorder:FindChild("BorderWidth"):FindChild("Output"):SetText(self.settings.nBorderWidth)
	self.wndOptionsBarBorder:FindChild("BorderOpacity"):FindChild("Slider"):SetValue(VILib:Round(self.settings.nBorderOpacity * 100))
	self.wndOptionsBarBorder:FindChild("BorderOpacity"):FindChild("Output"):SetText(VILib:Round(self.settings.nBorderOpacity * 100) .. "%")
	self.wndOptionsBarBorder:FindChild("BorderColor"):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.sBorder)
	self.wndOptionsBarBorder:FindChild("GraphicBorderShow"):FindChild("Button"):SetCheck(self.settings.bShowGraphicBorder)
	self.wndOptionsBarBorder:FindChild("GraphicBorderColor"):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.sGraphicBorder)

	self.wndOptionsBarFlash:FindChild("BarFlash"):FindChild("Button"):SetCheck(self.settings.bShowBarFlash)
	self.wndOptionsBarFlash:FindChild("BarFlashOnlyWhenReachingFull"):FindChild("Button"):SetCheck(self.settings.bOnlyShowWhenReachingFull)
	self.wndOptionsBarFlash:FindChild("BarFlashColor"):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.sBarFlash)
	self.wndOptionsBarFlash:FindChild("BarFlashOpacity"):FindChild("Slider"):SetValue(VILib:Round(self.settings.nBarFlashOpacity * 100))
	self.wndOptionsBarFlash:FindChild("BarFlashOpacity"):FindChild("Output"):SetText(VILib:Round(self.settings.nBarFlashOpacity * 100) .. "%")

	self.wndOptionsText:FindChild("TextShow"):FindChild("Button"):SetCheck(self.settings.tText.bShow)
	self.wndOptionsText:FindChild("Align"):FindChild("Slider"):SetValue(self.settings.tText.nAlign)
	self.wndOptionsText:FindChild("Align"):FindChild("Output"):SetText(arTextAligns[self.settings.tText.nAlign])
	self.wndOptionsText:FindChild("TextAlign"):FindChild("Slider"):SetValue(self.settings.tText.nTextAlign)
	self.wndOptionsText:FindChild("TextAlign"):FindChild("Output"):SetText(arTextAligns[self.settings.tText.nTextAlign])
	self.wndOptionsText:FindChild("TextScale"):FindChild("Slider"):SetValue(VILib:Round(self.settings.tText.nScale * 100))
	self.wndOptionsText:FindChild("TextScale"):FindChild("Output"):SetText(VILib:Round(self.settings.tText.nScale * 100) .. "%")
	self.wndOptionsText:FindChild("TextOffsetX"):FindChild("Slider"):SetValue(self.settings.tText.nOffsetX)
	self.wndOptionsText:FindChild("TextOffsetX"):FindChild("Output"):SetText(self.settings.tText.nOffsetX)
	self.wndOptionsText:FindChild("TextOffsetY"):FindChild("Slider"):SetValue(self.settings.tText.nOffsetY)
	self.wndOptionsText:FindChild("TextOffsetY"):FindChild("Output"):SetText(self.settings.tText.nOffsetY)
	self.wndOptionsText:FindChild("TextOpacity"):FindChild("Slider"):SetValue(VILib:Round(self.settings.tText.fOpacity * 100))
	self.wndOptionsText:FindChild("TextOpacity"):FindChild("Output"):SetText(VILib:Round(self.settings.tText.fOpacity * 100) .. "%")
	self.wndOptionsText:FindChild("TextColor"):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.sText)

	self.wndOptionsStealthed:FindChild("BarLookContainer:BarStylesButton:Button"):SetCheck(self.settings.tStealthed.bBarStyle)
	self.wndOptionsStealthed:FindChild("BarLookContainer:BarColor:Button"):SetCheck(self.settings.tStealthed.bBarColor)
	self.wndOptionsStealthed:FindChild("BarLookContainer:BarColor:ColorSwatchBG:ColorSwatch"):SetBGColor(self.settings.tColors.sStealthedBar)
	self.wndOptionsStealthed:FindChild("BarLookContainer:BarColor:2xButton"):SetCheck(self.settings.tColors.bStealthedBar2x)
	self.wndOptionsStealthed:FindChild("BarLookContainer:BackgroundBarColor:Button"):SetCheck(self.settings.tStealthed.bBGBarColor)
	self.wndOptionsStealthed:FindChild("BarLookContainer:BackgroundBarColor:ColorSwatchBG:ColorSwatch"):SetBGColor(self.settings.tColors.sStealthedBGBar)
	self.wndOptionsStealthed:FindChild("BarLookContainer:MasterOpacity:Button"):SetCheck(self.settings.tStealthed.bMasterOpacity)
	self.wndOptionsStealthed:FindChild("BarLookContainer:MasterOpacity:Slider"):SetValue(VILib:Round(self.settings.tStealthed.fMasterOpacity * 100))
	self.wndOptionsStealthed:FindChild("BarLookContainer:MasterOpacity:Output"):SetText(VILib:Round(self.settings.tStealthed.fMasterOpacity * 100) .. "%")
	self.wndOptionsStealthed:FindChild("BarLookContainer:HighlightOpacity:Button"):SetCheck(self.settings.tStealthed.bHighlightOpacity)
	self.wndOptionsStealthed:FindChild("BarLookContainer:HighlightOpacity:Slider"):SetValue(VILib:Round(self.settings.tStealthed.fHighlightOpacity * 100))
	self.wndOptionsStealthed:FindChild("BarLookContainer:HighlightOpacity:Output"):SetText(VILib:Round(self.settings.tStealthed.fHighlightOpacity * 100) .. "%")
	self.wndOptionsStealthed:FindChild("BarStylesButton"):FindChild("ColorSwatch"):SetBGColor(self.settings.tColors.sBarTest)
	self.wndOptionsStealthed:FindChild("BarStylesButton"):FindChild("2xButton"):SetCheck(self.settings.tColors.bBarTest2x)
	self.wndOptionsStealthed:FindChild("BorderContainer:BorderColor:Button"):SetCheck(self.settings.tStealthed.bBorderColor)
	self.wndOptionsStealthed:FindChild("BorderContainer:BorderColor:ColorSwatchBG:ColorSwatch"):SetBGColor(self.settings.tColors.sStealthedBorder)
	self.wndOptionsStealthed:FindChild("BorderContainer:BorderSize:Button"):SetCheck(self.settings.tStealthed.bBorderSize)
	self.wndOptionsStealthed:FindChild("BorderContainer:BorderSize:Slider"):SetValue(self.settings.tStealthed.nBorderWidth)
	self.wndOptionsStealthed:FindChild("BorderContainer:BorderSize:Output"):SetText(self.settings.tStealthed.nBorderWidth)
	self.wndOptionsStealthed:FindChild("BorderContainer:BorderOpacity:Button"):SetCheck(self.settings.tStealthed.bBorderOpacity)
	self.wndOptionsStealthed:FindChild("BorderContainer:BorderOpacity:Slider"):SetValue(VILib:Round(self.settings.tStealthed.fBorderOpacity * 100))
	self.wndOptionsStealthed:FindChild("BorderContainer:BorderOpacity:Output"):SetText(VILib:Round(self.settings.tStealthed.fBorderOpacity * 100) .. "%")
	self.wndOptionsStealthed:FindChild("TextContainer:TextColor:Button"):SetCheck(self.settings.tStealthed.bTextColor)
	self.wndOptionsStealthed:FindChild("TextContainer:TextColor:ColorSwatchBG:ColorSwatch"):SetBGColor(self.settings.tColors.sStealthedText)
	self.wndOptionsStealthed:FindChild("TextContainer:TextOpacity:Button"):SetCheck(self.settings.tStealthed.bTextOpacity)
	self.wndOptionsStealthed:FindChild("TextContainer:TextOpacity:Slider"):SetValue(VILib:Round(self.settings.tStealthed.fTextOpacity * 100))
	self.wndOptionsStealthed:FindChild("TextContainer:TextOpacity:Output"):SetText(VILib:Round(self.settings.tStealthed.fTextOpacity * 100) .. "%")
	self.wndOptionsStealthed:FindChild("PatternContainer:PatternShow:Button"):SetCheck(self.settings.tStealthed.bPattern)
	self.wndOptionsStealthed:FindChild("PatternContainer:PatternOpacity:Button"):SetCheck(self.settings.tStealthed.bPatternOpacity)
	self.wndOptionsStealthed:FindChild("PatternContainer:PatternOpacity:Slider"):SetValue(VILib:Round(self.settings.tStealthed.fPatternOpacity * 100))
	self.wndOptionsStealthed:FindChild("PatternContainer:PatternOpacity:Output"):SetText(VILib:Round(self.settings.tStealthed.fPatternOpacity * 100) .. "%")
	self.wndOptionsStealthed:FindChild("PatternContainer:PatternColor:Button"):SetCheck(self.settings.tStealthed.bPatternColor)
	self.wndOptionsStealthed:FindChild("PatternContainer:PatternColor:ColorSwatchBG:ColorSwatch"):SetBGColor(self.settings.tColors.sStealthedPattern)

	self.wndOptionsAdvanced:FindChild("FadeInTime"):FindChild("Slider"):SetValue(VILib:Round(self.settings.nFadeInTime, 1))
	self.wndOptionsAdvanced:FindChild("FadeInTime"):FindChild("Output"):SetText(VILib:Round(self.settings.nFadeInTime, 1, true) .. " sec")
	self.wndOptionsAdvanced:FindChild("FadeOutTime"):FindChild("Slider"):SetValue(VILib:Round(self.settings.nFadeOutTime, 1))
	self.wndOptionsAdvanced:FindChild("FadeOutTime"):FindChild("Output"):SetText(VILib:Round(self.settings.nFadeOutTime, 1, true) .. " sec")
	self.wndOptionsAdvanced:FindChild("FadeOutDelay"):FindChild("Slider"):SetValue(VILib:Round(self.settings.nFadeOutDelay / 1000, 1))
	self.wndOptionsAdvanced:FindChild("FadeOutDelay"):FindChild("Output"):SetText(VILib:Round(self.settings.nFadeOutDelay / 1000, 1, true) .. " sec")
	local sDisclaimer = ""
	for k, v in pairs(tSmoothUpdateDisclaimer) do
		sDisclaimer = sDisclaimer .. "<P>" .. v .. "</P><P TextColor=\"00ffffff\">-</P>"
	end
	sDisclaimer = string.format(sSmoothUpdateDisclaimer, sDisclaimer)
	self.wndOptionsAdvanced:FindChild("DisclaimerText"):SetAML(sDisclaimer)
	self.wndOptionsAdvanced:FindChild("UseSmoothUpdate"):FindChild("Button"):SetCheck(self.settings.bUseSmoothUpdate)
	self.wndOptionsAdvanced:FindChild("SmoothUpdateText"):FindChild("Button"):SetCheck(self.settings.bSmoothUpdateText)

--	self.wndOptionsAdvanced:FindChild("MarkerShow"):FindChild("Button"):SetCheck(self.settings.tMarkers.bShow)
end

function StalkerResourceSystem:FillAnchorList()
	local nSel = 0
	self.wndOptionsSizePosition:FindChild("AnchorParent:AnchorGrid"):DeleteAll()
	for i, v in ipairs(tAnchors) do
		if self.settings.tWindowPositions.nMainAnchor == i then
			self.wndOptionsSizePosition:FindChild("PositionContainer:Anchor:DropdownBtn"):SetText(v["sLabel"])
			nSel = i
		end
		self.wndOptionsSizePosition:FindChild("AnchorParent:AnchorGrid"):AddRow(v["sLabel"], "", v)
	end
	self.wndOptionsSizePosition:FindChild("AnchorParent:AnchorGrid"):SetCurrentRow(nSel)
end

function StalkerResourceSystem:ShowHideBarStyleOptions(wndHandler, wndControl, eMouseButton)
	if not self.wndBarStyles then
		self.wndBarStyles = Apollo.LoadForm(self.xmlDoc, "BarStylesForm", nil, self)
		local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
		local nLeft, nTop, nRight, nBottom = self.wndBarStyles:GetRect()
		local nWidth, nHeight = nRight - nLeft, nBottom - nTop
		if self.settings.tWindowPositions.tBarStyleOptions[1] then
			self.wndBarStyles:Move(VILib:Round(nScreenWidth * self.settings.tWindowPositions.tBarStyleOptions[1]), VILib:Round(nScreenHeight * self.settings.tWindowPositions.tBarStyleOptions[2]), nWidth, nHeight)
		else
			self.wndBarStyles:Move(VILib:Round((nScreenWidth / 2) - (nWidth / 2)), VILib:Round((nScreenHeight / 2) - (nHeight / 2)), nWidth, nHeight)
		end
		self.wndBarStyles:ToFront()
		self.wndBarStyles:Show(true, false)
		Apollo.CreateTimer("CreateBarStylesTimer", 0.3, false)
	else
		self.wndBarStyles:Show(false, false)
	end
end

function StalkerResourceSystem:OnCreateBarStylesTimer()
	self:CreateBarStyles()
end

function StalkerResourceSystem:OnBarFlashColorPickerTimer()
	if not ColorPicker.window:IsVisible() then
		self.bForceBarFlash = false
		self.bForceBarFlashColorPickerOpen = false
		self.wndResourceBar:FindChild("BarFlash"):Show(false, false)
		Apollo.CreateTimer("StopBFCPTimer", 0.1, false)
	end
end

function StalkerResourceSystem:CreateBarStyles()
	self.wndBarStylesList = Apollo.LoadForm(self.xmlDoc, "BarStylesList", self.wndBarStyles:FindChild("ContentFrame"), self)
	self.tBarStyles = {}
	local nYOffset = 0
	local nSelBarStyle = self.settings.nBarStyle
	if self.bBarStyleStealthed then
		nSelBarStyle = self.settings.tStealthed.nBarStyle
	end
	for i = 1, nBarStyles, 1 do
		self.tBarStyles[i] = Apollo.LoadForm(self.xmlDoc, "BarStyleContainer", self.wndBarStylesList, self)
		self.tBarStyles[i]:FindChild("HoverGlow"):SetSprite("VSRS_sprites:BarSelectedGlow")
		self.tBarStyles[i]:FindChild("SelectedGlow"):SetSprite("VSRS_sprites:BarSelectedGlow")
		self.tBarStyles[i]:FindChild("BarSprite"):SetSprite("VSRS_sprites:ProgressBarH" .. i)
		self.tBarStyles[i]:FindChild("BarSprite"):SetBGColor(self:GetColor("BarTest"))
		if not tNoHighlight[i] then
			self.tBarStyles[i]:FindChild("BarHighlightSprite"):SetSprite("VSRS_sprites:ProgressBarH" .. i .. "_Hi")
			self.tBarStyles[i]:FindChild("BarHighlightSprite"):SetOpacity(self.settings.nBarHighlightOpacity, 1000)
		end
		self.tBarStyles[i]:FindChild("HoverGlow"):SetOpacity(0, 100)
		if i == nSelBarStyle then
			self.tBarStyles[i]:FindChild("SelectedGlow"):SetOpacity(0.9, 1)
		else
			self.tBarStyles[i]:FindChild("SelectedGlow"):SetOpacity(0, 100)
		end
		self.tBarStyles[i]:SetName(i)
		if i % 2 == 1 then
			self.tBarStyles[i]:Move(0, nYOffset, 280, 28)
		else
			self.tBarStyles[i]:Move(300, nYOffset, 280, 28)
			nYOffset = nYOffset + 48
		end
	end
	self.wndBarStylesList:SetAnchorOffsets(10, 20, 600, nYOffset + 40)
	self.wndBarStyles:FindChild("ContentFrame"):RecalculateContentExtents()
	self.wndBarStyles:FindChild("LoadingText"):Show(false, false, 1)
end

function StalkerResourceSystem:ShowHideFontOptions()
	if not self.wndFontOptions then
		self.wndFontOptions = Apollo.LoadForm(self.xmlDoc, "FontOptionsForm", nil, self)
		self.wndFontOptionsList = Apollo.LoadForm(self.xmlDoc, "FontOptionsList", self.wndFontOptions:FindChild("ContentFrame"), self)
		self.tFontOptions = {}
		local nYOffset = 0
		local i = 1
		for k, v in pairs(tFonts) do
			self.tFontOptions[i] = Apollo.LoadForm(self.xmlDoc, "FontOptionContainer", self.wndFontOptionsList, self)
			self.tFontOptions[i]:FindChild("HoverGlow"):SetSprite("VSRS_sprites:BarSelectedGlow")
			self.tFontOptions[i]:FindChild("SelectedGlow"):SetSprite("VSRS_sprites:BarSelectedGlow")
			self.tFontOptions[i]:FindChild("Display"):SetFont(v[1])
			self.tFontOptions[i]:FindChild("Display"):SetText(v[2])
			self.tFontOptions[i]:FindChild("HoverGlow"):SetOpacity(0, 100)
			if i == self.settings.tText.nFont then
				self.tFontOptions[i]:FindChild("SelectedGlow"):SetOpacity(0.9, 1)
			else
				self.tFontOptions[i]:FindChild("SelectedGlow"):SetOpacity(0, 100)
			end
			self.tFontOptions[i]:SetName(i)
			self.tFontOptions[i]:Move(0, nYOffset, 221, 40)
			nYOffset = nYOffset + 50
			i = i + 1
		end
		self.wndFontOptionsList:SetAnchorOffsets(10, 20, 241, nYOffset + 40)
		self.wndFontOptions:FindChild("ContentFrame"):RecalculateContentExtents()
		self.wndFontOptions:ToFront()
		local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
		local nLeft, nTop, nRight, nBottom = self.wndFontOptions:GetRect()
		local nWidth, nHeight = nRight - nLeft, nBottom - nTop
		if self.settings.tWindowPositions.tFontOptions[1] then
			self.wndFontOptions:Move(VILib:Round(nScreenWidth * self.settings.tWindowPositions.tFontOptions[1]), VILib:Round(nScreenHeight * self.settings.tWindowPositions.tFontOptions[2]), nWidth, nHeight)
		else
			self.wndFontOptions:Move(VILib:Round((nScreenWidth / 2) - (nWidth / 2)), VILib:Round((nScreenHeight / 2) - (nHeight / 2)), nWidth, nHeight)
		end
		self.wndFontOptions:Show(true, false)
	else
		self.wndFontOptions:Show(false, false)
	end
end

function StalkerResourceSystem:ShowHideMarkerSystem()
	VILib:WriteToChat("This feature is not implemented yet.")
	if not self.wndMarkerSystem then
		self.wndMarkerSystem = Apollo.LoadForm(self.xmlDoc, "MarkerSystemForm", nil, self)
		self.wndMarkerSystemList = Apollo.LoadForm(self.xmlDoc, "MarkerList", self.wndMarkerSystem:FindChild("MarkerContentList"), self)
		self.tMarkerSystemItems = {}
		local nYOffset = 0
		local i = 1
--		for k, v in pairs(tFonts) do
			self.tMarkerSystemItems[i] = Apollo.LoadForm(self.xmlDoc, "MarkerListItemForm", self.wndMarkerSystemList, self)
			self.tMarkerSystemItems[i]:SetName(i)
--			self.tFontOptions[i]:Move(0, nYOffset, 221, 40)
			nYOffset = nYOffset + 28
			i = i + 1
--		end
		self.wndMarkerSystemList:SetAnchorOffsets(0, 0, 344, nYOffset + 64)
		self.wndMarkerSystem:FindChild("MarkerContentList"):RecalculateContentExtents()
		self.wndMarkerSystem:ToFront()
		local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
		local nLeft, nTop, nRight, nBottom = self.wndMarkerSystem:GetRect()
		local nWidth, nHeight = nRight - nLeft, nBottom - nTop
		if self.settings.tWindowPositions.tMarkerSystem[1] then
			self.wndMarkerSystem:Move(VILib:Round(nScreenWidth * self.settings.tWindowPositions.tMarkerSystem[1]), VILib:Round(nScreenHeight * self.settings.tWindowPositions.tMarkerSystem[2]), nWidth, nHeight)
		else
			self.wndMarkerSystem:Move(VILib:Round((nScreenWidth / 2) - (nWidth / 2)), VILib:Round((nScreenHeight / 2) - (nHeight / 2)), nWidth, nHeight)
		end
		self.wndMarkerSystem:Show(true, false)
	else
		self.wndMarkerSystem:Show(false, false)
	end
end

function StalkerResourceSystem:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account then return nil end
	self.settings.bFirstRun = false
	return self.settings
end

function StalkerResourceSystem:OnRestore(eLevel, tData)
	if tData then
		self.settings = VILib:MergeTables(self.settings, tData)
		if self.bCharacterCreated then
			self:CreateResourceBar()
			self:SetAllOptions()
			if self.settings.bAlwaysShow then self.wndResourceContainer:Show(true, true) else self.wndResourceContainer:Show(false, true) end
		end
	end
end

function StalkerResourceSystem:OnResourceBarMove(wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom)
	if self.settings.bPositionOnChar then
		local nL, nT, nR, nB = self.wndResourceContainer:GetAnchorOffsets()
		local nWidth, nHeight = VILib:Round((nR - nL) / 2), VILib:Round((nB - nT) / 2)
		local fScale = 1 - (1 / Apollo.GetConsoleVariable("ui.Scale"))
		local nOffsetX, nOffsetY = VILib:Round(math.abs((nWidth * 2) * fScale) / 2), VILib:Round(math.abs((nHeight * 2) * fScale) / 2)
		self.settings.tWindowPositions.tMainCharOffsets[1] = nL + nWidth + nOffsetX
		self.settings.tWindowPositions.tMainCharOffsets[2] = nT + nHeight + nOffsetY
		if self.wndOptions then
			self.wndOptionsSizePosition:FindChild("CharacterPosContainer:PosOffsetX:Slider"):SetValue(self.settings.tWindowPositions.tMainCharOffsets[1])
			self.wndOptionsSizePosition:FindChild("CharacterPosContainer:PosOffsetX:Output"):SetText(self.settings.tWindowPositions.tMainCharOffsets[1])
			self.wndOptionsSizePosition:FindChild("CharacterPosContainer:PosOffsetY:Slider"):SetValue(self.settings.tWindowPositions.tMainCharOffsets[2])
			self.wndOptionsSizePosition:FindChild("CharacterPosContainer:PosOffsetY:Output"):SetText(self.settings.tWindowPositions.tMainCharOffsets[2])
		end
	else
		local nL, nT, nR, nB = self.wndResourceContainer:GetAnchorOffsets()
		local nWidth, nHeight = VILib:Round((nR - nL) / 2), VILib:Round((nB - nT) / 2)
		local fScale = 1 - (1 / Apollo.GetConsoleVariable("ui.Scale"))
		local nOffsetX, nOffsetY = VILib:Round(math.abs((nWidth * 2) * fScale) / 2), VILib:Round(math.abs((nHeight * 2) * fScale) / 2)
		self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][1] = nL + nWidth + nOffsetX
		self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][2] = nT + nHeight + nOffsetY
		if self.wndOptions then
			self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetX:Slider"):SetValue(self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][1])
			self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetX:Output"):SetText(self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][1])
			self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetY:Slider"):SetValue(self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][2])
			self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetY:Output"):SetText(self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][2])
		end
	end
end

function StalkerResourceSystem:OnResourceBarMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
--	if eMouseButton == 1 then
--		if Apollo.IsShiftKeyDown() then
--			if self.settings.bPositionOnChar then
--				local nLeft, tmp, nRight, tmp = self.wndResourceBar:GetRect()
--				local tmp, nTop, tmp, nBottom = self.wndResourceBar:GetAnchorOffsets()
--				local nWidth = (nRight - nLeft) / 2
--				self.wndResourceBar:SetAnchorOffsets(-nWidth, nTop, nWidth, nBottom)
--			else
--				local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
--				local nLeft, nTop, nRight, nBottom = self.wndResourceBar:GetRect()
--				local nWidth = nRight - nLeft
--				local nHeight = nBottom - nTop
--				local nPosLeft, nPosTop = self.wndResourceBar:GetPos()
--				self.wndResourceBar:Move((nScreenWidth / 2) - (nWidth / 2), nPosTop, nWidth, nHeight)
--			end
--			StalkerResourceSystem:OnResourceBarMove()
--		end
--	end
end

function StalkerResourceSystem:HasColorPicker()
	if ColorPicker then
		return true
	end
	VILib:WriteToChat("To change colors you need to have the ColorPicker add-on installed.")
	return false
end


---------------------------------------------------------------------------------------------------
-- OptionsForm Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnOptionsWindowClosed(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self:ShowHideOptions()
end

function StalkerResourceSystem:OnOptionsWindowHide(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	if self.wndOptions then
		self.wndOptions:Destroy()
		self.wndOptions = nil
		OptionsWindowRef = nil
		SettingsRef = nil
		ResourceBarRef = nil
	end
end

function StalkerResourceSystem:OnOptionsWindowMove(wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom)
	local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
	local nPosLeft, nPosTop = self.wndOptions:GetPos()
	self.settings.tWindowPositions.tOptions[1], self.settings.tWindowPositions.tOptions[2] = nPosLeft / nScreenWidth, nPosTop / nScreenHeight
end

function StalkerResourceSystem:OnCloseButton(wndHandler, wndControl, eMouseButton)
	self:ShowHideOptions()
end

function StalkerResourceSystem:OnOptionsMenuButtonCheck(wndHandler, wndControl, eMouseButton)
	local nOption = self.wndOptions:FindChild("Menu"):GetRadioSel("VSRSOptionsGroup")
	if nOption == 0 then
		self.wndOptions:FindChild("Menu:SizePositionButton"):SetCheck(true)
	end
	
	self.wndOptionsSizePosition:Show(nOption <= 1)
	self.wndOptionsBarLook:Show(nOption == 2)
	self.wndOptionsBarPattern:Show(nOption == 3)
	self.wndOptionsBarBorder:Show(nOption == 4)
	self.wndOptionsBarFlash:Show(nOption == 5)
	self.wndOptionsText:Show(nOption == 6)
	self.wndOptionsStealthed:Show(nOption == 7)
	self.wndOptionsSpecials:Show(nOption == 8)
	self.wndOptionsAdvanced:Show(nOption == 9)

	self.wndOptions:FindChild("ContentFrame"):SetVScrollPos(0)
	self.wndOptions:FindChild("ContentFrame"):RecalculateContentExtents()
end


---------------------------------------------------------------------------------------------------
-- OptionsList Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnBarWidthSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nBarWidth = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetBarSize()
end

function StalkerResourceSystem:OnBarHeightSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nBarHeight = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetBarSize()
end

function StalkerResourceSystem:OnBorderWidthSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nBorderWidth = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetBorderSettings()
end

function StalkerResourceSystem:OnAlignSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tText.nAlign = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(arTextAligns[fNewValue])
	self:SetFontSettings()
end

function StalkerResourceSystem:OnTextAlignSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tText.nTextAlign = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(arTextAligns[fNewValue])
	self.wndResourceBar:FindChild("ProgressBarText"):SetTextFlags("DT_CENTER", false)
	self.wndResourceBar:FindChild("ProgressBarText"):SetTextFlags("DT_RIGHT", false)
	if fNewValue > 1 then
		self.wndResourceBar:FindChild("ProgressBarText"):SetTextFlags("DT_" .. arTextAligns[fNewValue]:upper(), true)
	end
end

function StalkerResourceSystem:OnTextShowButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tText.bShow = wndControl:IsChecked()
	self.wndResourceBar:FindChild("ProgressBarText"):Show(self.settings.tText.bShow, true)
end

function StalkerResourceSystem:OnTextColorSwatchMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sText, false, UpdateTextColor)
	end
end

function UpdateTextColor()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sText = VILib:ConvertColorToHex(tColors.sText)
	OptionsWindowRef:FindChild("TextColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sText)
	ResourceBarRef:FindChild("ProgressBarText"):SetTextColor(SettingsRef.tColors.sText)
end

function StalkerResourceSystem:OnTestColorSwatchMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sBarTest, false, UpdateBarTestColor)
	end
end

function UpdateBarTestColor()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sBarTest = VILib:ConvertColorToHex(tColors.sBarTest)
	OptionsWindowRef:FindChild("BarStylesButton"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sBarTest)
	StalkerResourceSystem:UpdateBarStyleColors()
end

function StalkerResourceSystem:UpdateBarStyleColors()
	if self.wndBarStyles then
		for k, v in pairs(self.tBarStyles) do
			v:FindChild("BarSprite"):SetBGColor(self:GetColor("BarTest"))
		end
	end
end

function StalkerResourceSystem:OnMasterOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nMasterOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self.wndResourceContainer:SetOpacity(fNewValue / 100, 1000)
end

function StalkerResourceSystem:UpdateBarStyleHighlights()
	if self.wndBarStyles then
		for k, v in pairs(self.tBarStyles) do
			v:FindChild("BarHighlightSprite"):SetOpacity(self.settings.nBarHighlightOpacity, 1000)
		end
	end
end

function StalkerResourceSystem:OnBarTestColorX2MouseUp(wndHandler, wndControl, eMouseButton)
	self.settings.tColors.bBarTest2x = wndControl:IsChecked()
	self:UpdateBarStyleColors()
end

function StalkerResourceSystem:OnHighlightOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nBarHighlightOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:SetBarHighlightOpacity()
	self:UpdateBarStyleHighlights()
end

function StalkerResourceSystem:OnTextScaleSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tText.nScale = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:SetFontSettings()
end

function StalkerResourceSystem:OnTextOffsetXSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tText.nOffsetX = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetFontSettings()
end

function StalkerResourceSystem:OnTextOffsetYSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tText.nOffsetY = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetFontSettings()
end

function StalkerResourceSystem:OnTextOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue, bOkToChange)
	self.settings.tText.fOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:SetFontSettings()
end

function StalkerResourceSystem:OnColorModeCheckboxUp(wndHandler, wndControl, eMouseButton)
	self.settings.nColorMode = tonumber(wndControl:GetData())
	if self.settings.nColorMode == 3 then self.nCurrentStance = 0 end
end

function StalkerResourceSystem:OnBackgroundSameAsMainButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bBackgroundColorSameAsBar = wndControl:IsChecked()
	if self.settings.nColorMode == 3 then self.nCurrentStance = 0 end
end

function StalkerResourceSystem:OnFlashBarButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bShowBarFlash = wndControl:IsChecked()
end

function StalkerResourceSystem:OnBarFlashOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nBarFlashOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:SetBarFlashOptions()
end

function StalkerResourceSystem:OnBarFlashColorButtonUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sBarFlash, false, UpdateBarFlashColor)
		self.bForceBarFlashColorPickerOpen = true
		self.bForceBarFlash = true
		Apollo.CreateTimer("BarFlashColorPickerTimer", 0.1, true)
	end
end

function UpdateBarFlashColor()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sBarFlash = VILib:ConvertColorToHex(tColors.sBarFlash)
	OptionsWindowRef:FindChild("BarFlashColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sBarFlash)
	ResourceBarRef:FindChild("BarFlash"):SetBGColor(SettingsRef.tColors.sBarFlash)
end

function StalkerResourceSystem:OnBarColorButtonUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sBar, false, UpdateBarColor)
	end
end

function UpdateBarColor()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sBar = VILib:ConvertColorToHex(tColors.sBar)
	OptionsWindowRef:FindChild("ColorMode1Constant"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sBar)
end

function StalkerResourceSystem:OnTransitionColorIntensitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nTransitionColorIntensity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
end

function StalkerResourceSystem:OnTransitionColorIntensity2xButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bTransitionColor2x = wndControl:IsChecked()
end

function StalkerResourceSystem:OnColorMode3StanceMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		nStanceColorIndex = tonumber(wndControl:GetParent():GetData())
		if nStanceColorIndex and nStanceColorIndex > 0 then
			ColorPicker.AdjustCColor(tColors.tStances[nStanceColorIndex], false, UpdateStanceColor)
		end
	end
end

function UpdateStanceColor()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.tStances[nStanceColorIndex] = VILib:ConvertColorToHex(tColors.tStances[nStanceColorIndex])
	OptionsWindowRef:FindChild("ColorMode3Stance" .. nStanceColorIndex):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.tStances[nStanceColorIndex])
	if SettingsRef.nColorMode == 3 then StalkerResourceSystem.nCurrentStance = 0 end
end

function StalkerResourceSystem:OnColorMode3Stance2xButtonUp(wndHandler, wndControl, eMouseButton)
	local n = tonumber(wndControl:GetParent():GetData())
	if n and n > 0 then
		self.settings.tColors.tStances2x[n] = wndControl:IsChecked()
		if self.settings.nColorMode == 3 then self.nCurrentStance = 0 end
	end
end

function StalkerResourceSystem:OnBackgroundBarColorMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sBackgroundBar, false, UpdateBackgroundBarColor)
	end
end

function UpdateBackgroundBarColor()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sBackgroundBar = VILib:ConvertColorToHex(tColors.sBackgroundBar)
	OptionsWindowRef:FindChild("BackgroundBarColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sBackgroundBar)
end

function StalkerResourceSystem:OnBackgroundBarColor2xButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tColors.bBackgroundBar2x = wndControl:IsChecked()
end

function StalkerResourceSystem:OnFadeInTimeSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nFadeInTime = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(VILib:Round(fNewValue, 1, true) .. " sec")
end

function StalkerResourceSystem:OnFadeOutTimeSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nFadeOutTime = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(VILib:Round(fNewValue, 1, true) .. " sec")
end

function StalkerResourceSystem:OnFadeOutDelaySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nFadeOutDelay = fNewValue * 1000
	wndControl:GetParent():FindChild("Output"):SetText(VILib:Round(fNewValue, 1, true) .. " sec")
end

function StalkerResourceSystem:OnBackgroundBarOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nBackgroundBarOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self.wndResourceBar:FindChild("ProgressBarBG"):SetOpacity(self.settings.nBackgroundBarOpacity, 100)
end

function StalkerResourceSystem:OnAlwaysShowButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bAlwaysShow = wndControl:IsChecked()
end

function StalkerResourceSystem:OnMarkersShowButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tMarkers.bShow = wndControl:IsChecked()
end

function StalkerResourceSystem:OnColorMode1Color2xButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tColors.bBar2x = wndControl:IsChecked()
end

function StalkerResourceSystem:OnUseSmoothUpdateButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bUseSmoothUpdate = wndControl:IsChecked()
end

function StalkerResourceSystem:OnSmoothUpdateTextButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bSmoothUpdateText = wndControl:IsChecked()
end

function StalkerResourceSystem:OnBackgroundOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nBackgroundOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self.wndResourceBar:FindChild("Background"):SetOpacity(self.settings.nBackgroundOpacity, 100)
end

function StalkerResourceSystem:OnBorderColorButtonUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sBorder, false, UpdateBorderColor)
	end
end

function UpdateBorderColor()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sBorder = VILib:ConvertColorToHex(tColors.sBorder)
	OptionsWindowRef:FindChild("BorderColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sBorder)
	StalkerResourceSystem:SetBorderSettings()
end

function StalkerResourceSystem:OnBorderOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nBorderOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:SetBorderSettings()
end

function StalkerResourceSystem:OnBackgroundColorMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sBackground, false, UpdateBackgroundColor)
	end
end

function UpdateBackgroundColor()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sBackground = VILib:ConvertColorToHex(tColors.sBackground)
	OptionsWindowRef:FindChild("BackgroundColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sBackground)
	StalkerResourceSystem:SetBarStyle()
end

function StalkerResourceSystem:OnFlashOnlyWhenReachingFullButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bOnlyShowWhenReachingFull = wndControl:IsChecked()
end

function StalkerResourceSystem:ForceBarFlashOn(wndHandler, wndControl, x, y)
	self.bForceBarFlash = true
end

function StalkerResourceSystem:ForceBarFlashOff(wndHandler, wndControl, x, y)
	self.bForceBarFlash = false
	if not self.bForceFlashMouseButtonDown and not self.bForceBarFlashColorPickerOpen then self.wndResourceBar:FindChild("BarFlash"):Show(false, false) end
end

function StalkerResourceSystem:ForceBarFlashMouseDown(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation)
	self.bForceFlashMouseButtonDown = true
end

function StalkerResourceSystem:ForceBarFlashMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	self.bForceFlashMouseButtonDown = false
	if not self.bForceFlash and not self.bForceBarFlashColorPickerOpen then self.wndResourceBar:FindChild("BarFlash"):Show(false, false) end
end

function StalkerResourceSystem:OnTiePositionToCharacterButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bPositionOnChar = wndControl:IsChecked()
	self:CreateResourceBar()
	self:SetPosition()
end

function StalkerResourceSystem:OnVerticalPositionOnCharacterSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.fVerticalPositionOnChar = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(arPositionOnCharacterNames[fNewValue + 1])
	self:SetPosition()
end

function StalkerResourceSystem:OnMainAnchorButtonCheck(wndHandler, wndControl, eMouseButton)
	self.wndOptionsSizePosition:FindChild("AnchorParent"):Show(not self.bMainAnchorShown, true)
end

function StalkerResourceSystem:OnMainAnchorGridSelChange(wndHandler, wndControl, nRow)
	local tData = wndControl:GetCellData(nRow, 1)
	if not tData then return end
	self.wndOptionsSizePosition:FindChild("AnchorParent"):Show(false)
	self.wndOptionsSizePosition:FindChild("PositionContainer:Anchor:DropdownBtn"):SetText(tData["sLabel"])
	self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetX:Slider"):SetValue(self.settings.tWindowPositions.tMainOffsets[nRow][1])
	self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetX:Output"):SetText(self.settings.tWindowPositions.tMainOffsets[nRow][1])
	self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetY:Slider"):SetValue(self.settings.tWindowPositions.tMainOffsets[nRow][2])
	self.wndOptionsSizePosition:FindChild("PositionContainer:PosOffsetY:Output"):SetText(self.settings.tWindowPositions.tMainOffsets[nRow][2])
	self.settings.tWindowPositions.nMainAnchor = nRow
	self:SetPosition()
end

function StalkerResourceSystem:OnMouseDownCatcher(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation)
end

function StalkerResourceSystem:OnMainAnchorShow(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self:FillAnchorList()
	self.bMainAnchorShown = true
	self.wndOptionsSizePosition:FindChild("PositionContainer:Anchor:DropdownBtnBG"):SetCheck(true)
	self.wndOptionsSizePosition:FindChild("PositionContainer:Anchor:DropdownBtn:Arrow"):SetSprite("CRB_Basekit:kitIcon_Holo_UpArrow")
	self.wndOptions:SetStyle("Escapable", false)
end

function StalkerResourceSystem:OnMainAnchorClosed(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self.bMainAnchorShown = false
	self.wndOptionsSizePosition:FindChild("PositionContainer:Anchor:DropdownBtnBG"):SetCheck(false)
	self.wndOptionsSizePosition:FindChild("PositionContainer:Anchor:DropdownBtn:Arrow"):SetSprite("CRB_Basekit:kitIcon_Holo_DownArrow")
	Apollo.CreateTimer("MainEscapableTimer", 0.1, false)
end

function StalkerResourceSystem:OnMainAnchorButtonEnter(wndHandler, wndControl, x, y)
	self.wndOptionsSizePosition:FindChild("AnchorParent"):SetStyle("CloseOnExternalClick", false)
end

function StalkerResourceSystem:OnMainAnchorButtonExit(wndHandler, wndControl, x, y)
	self.wndOptionsSizePosition:FindChild("AnchorParent"):SetStyle("CloseOnExternalClick", true)
end

function StalkerResourceSystem:OnMainEscapableTimer()
	self.wndOptions:SetStyle("Escapable", true)
end


---------------------------------------------------------------------------------------------------
-- BarStylesForm Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnBarStyleOptionsWindowClosed(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self:ShowHideBarStyleOptions()
end

function StalkerResourceSystem:OnBarStyleOptionsWindowHide(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	if self.wndBarStyles then
		self.wndBarStylesList:Destroy()
		self.wndBarStylesList = nil
		self.wndBarStyles:Destroy()
		self.wndBarStyles = nil
	end
end

function StalkerResourceSystem:OnBarStyleOptionsWindowMove(wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom)
	local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
	local nPosLeft, nPosTop = self.wndBarStyles:GetPos()
	self.settings.tWindowPositions.tBarStyleOptions[1], self.settings.tWindowPositions.tBarStyleOptions[2] = nPosLeft / nScreenWidth, nPosTop / nScreenHeight
end

function StalkerResourceSystem:OnBarStyleOptionsCloseButton(wndHandler, wndControl, eMouseButton)
	self:ShowHideBarStyleOptions()
end


---------------------------------------------------------------------------------------------------
-- BarStyleContainer Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnBarStyleMouseEnter(wndHandler, wndControl, x, y)
	wndControl:GetParent():FindChild("HoverGlow"):SetOpacity(0.9, 10)
end

function StalkerResourceSystem:OnBarStyleMouseExit(wndHandler, wndControl, x, y)
	wndControl:GetParent():FindChild("HoverGlow"):SetOpacity(0, 2)
end

function StalkerResourceSystem:OnBarStyleMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self.bBarStyleStealthed then
		self.tBarStyles[self.settings.tStealthed.nBarStyle]:FindChild("SelectedGlow"):SetOpacity(0, 2)
		self.settings.tStealthed.nBarStyle = tonumber(wndControl:GetParent():GetName())
		if self.bIsStealthed then self:SetStealthedBarStyle() end
	else
		self.tBarStyles[self.settings.nBarStyle]:FindChild("SelectedGlow"):SetOpacity(0, 2)
		self.settings.nBarStyle = tonumber(wndControl:GetParent():GetName())
		self:SetBarStyle()
		self:SetBarHighlight()
	end
	wndControl:GetParent():FindChild("SelectedGlow"):SetOpacity(0.9, 7)
end


---------------------------------------------------------------------------------------------------
-- FontOptionsForm Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnFontOptionsWindowClosed(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self:ShowHideFontOptions()
end

function StalkerResourceSystem:OnFontOptionsWindowHide(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self.wndFontOptionsList:Destroy()
	self.wndFontOptionsList = nil
	self.wndFontOptions:Destroy()
	self.wndFontOptions = nil
end

function StalkerResourceSystem:OnFontOptionsWindowMove(wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom)
	local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
	local nPosLeft, nPosTop = self.wndFontOptions:GetPos()
	self.settings.tWindowPositions.tFontOptions[1], self.settings.tWindowPositions.tFontOptions[2] = nPosLeft / nScreenWidth, nPosTop / nScreenHeight
end

function StalkerResourceSystem:OnFontOptionsCloseButton(wndHandler, wndControl, eMouseButton)
	self:ShowHideFontOptions()
end


---------------------------------------------------------------------------------------------------
-- FontOptionContainer Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnFontOptionMouseEnter(wndHandler, wndControl, x, y)
	wndControl:GetParent():FindChild("HoverGlow"):SetOpacity(0.9, 10)
end

function StalkerResourceSystem:OnFontOptionMouseExit(wndHandler, wndControl, x, y)
	wndControl:GetParent():FindChild("HoverGlow"):SetOpacity(0, 2)
end

function StalkerResourceSystem:OnFontOptionMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	self.tFontOptions[self.settings.tText.nFont]:FindChild("SelectedGlow"):SetOpacity(0, 2)
	self.settings.tText.nFont = tonumber(wndControl:GetParent():GetName())
	wndControl:GetParent():FindChild("SelectedGlow"):SetOpacity(0.9, 7)
	self:SetFontSettings()
end


---------------------------------------------------------------------------------------------------
-- MarkerSystemForm Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnMarkerSystemWindowClosed(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self:ShowHideMarkerSystem()
end

function StalkerResourceSystem:OnMarkerSystemWindowHide(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	if self.wndMarkerSystem then
		self.wndMarkerSystemList:Destroy()
		self.wndMarkerSystemList = nil
		self.wndMarkerSystem:Destroy()
		self.wndMarkerSystem = nil
	end
end

function StalkerResourceSystem:OnMarkerSystemWindowMove(wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom)
	local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
	local nPosLeft, nPosTop = self.wndMarkerSystem:GetPos()
	self.settings.tWindowPositions.tMarkerSystem[1], self.settings.tWindowPositions.tMarkerSystem[2] = nPosLeft / nScreenWidth, nPosTop / nScreenHeight
end

function StalkerResourceSystem:OnMarkerSystemCloseButton(wndHandler, wndControl, eMouseButton)
	self:ShowHideMarkerSystem()
end


---------------------------------------------------------------------------------------------------
-- OptionsSizePositioning Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnOptionsSizePositioningShow(wndHandler, wndControl)
	local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
	local nHeight = 80
	local nL, nT, nR, nB = wndControl:FindChild("SizeContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	nL, nT, nR, nB = wndControl:FindChild("PositionContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	nL, nT, nR, nB = wndControl:FindChild("CharacterPosContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	wndControl:SetAnchorOffsets(0, 0, 0, nHeight)
	wndControl:FindChild("PositionContainer:PosOffsetX:Slider"):SetMinMax(-nScreenWidth, nScreenWidth)
	wndControl:FindChild("PositionContainer:PosOffsetY:Slider"):SetMinMax(-nScreenHeight, nScreenHeight)
	wndControl:FindChild("CharacterPosContainer:PosOffsetX:Slider"):SetMinMax(-nScreenWidth, nScreenWidth)
	wndControl:FindChild("CharacterPosContainer:PosOffsetY:Slider"):SetMinMax(-nScreenHeight, nScreenHeight)
end

function StalkerResourceSystem:OnOptionsSizePositioningHide(wndHandler, wndControl)
	wndControl:SetAnchorOffsets(0, 0, 0, 0)
end

function StalkerResourceSystem:OnPosOffsetXSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][1] = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetPosition()
end

function StalkerResourceSystem:OnPosOffsetYSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tWindowPositions.tMainOffsets[self.settings.tWindowPositions.nMainAnchor][2] = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetPosition()
end

function StalkerResourceSystem:OnCharPosOffsetXSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tWindowPositions.tMainCharOffsets[1] = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetPosition()
end

function StalkerResourceSystem:OnCharPosOffsetYSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tWindowPositions.tMainCharOffsets[2] = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetPosition()
end


---------------------------------------------------------------------------------------------------
-- OptionsBarLook Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnOptionsBarLookShow(wndHandler, wndControl)
	local nHeight = 70
	local nL, nT, nR, nB = wndControl:FindChild("BarLookContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	nL, nT, nR, nB = wndControl:FindChild("BarColoringContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	wndControl:SetAnchorOffsets(0, 0, 0, nHeight)
end

function StalkerResourceSystem:OnOptionsBarLookHide(wndHandler, wndControl)
	wndControl:SetAnchorOffsets(0, 0, 0, 0)
end

function StalkerResourceSystem:OpenBarStyleOptions( wndHandler, wndControl, eMouseButton )
	self.bBarStyleStealthed = false
	self:ShowHideBarStyleOptions()
end


---------------------------------------------------------------------------------------------------
-- OptionsBarPattern Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnOptionsBarPatternShow(wndHandler, wndControl)
	local nHeight = 70
	local nL, nT, nR, nB = wndControl:FindChild("BarPatternContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	nL, nT, nR, nB = wndControl:FindChild("AvailablePatternsContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	wndControl:SetAnchorOffsets(0, 0, 0, nHeight)

	if not self.tPatterns then
		self.tPatterns = {}
		local nYOffset = -60
		local nXOffset = 160
		local nSelPattern = self.settings.nPatternID
		if self.bBarStyleStealthed then
			nSelPattern = self.settings.tStealthed.nPatternID
		end
		for i = 1, nPatterns, 1 do
			self.tPatterns[i] = Apollo.LoadForm(self.xmlDoc, "PatternTemplate", wndControl:FindChild("AvailablePatternsContainer:Patterns"), self)
			self.tPatterns[i]:FindChild("HoverGlow"):SetSprite("VSRS_sprites:BarSelectedGlow")
			self.tPatterns[i]:FindChild("SelectedGlow"):SetSprite("VSRS_sprites:BarSelectedGlow")
			self.tPatterns[i]:FindChild("Pattern"):SetSprite("VSRS_sprites:Pattern" .. i)
			self.tPatterns[i]:FindChild("HoverGlow"):SetOpacity(0, 100)
			if i == nSelPattern then
				self.tPatterns[i]:FindChild("SelectedGlow"):SetOpacity(0.9, 1)
			else
				self.tPatterns[i]:FindChild("SelectedGlow"):SetOpacity(0, 100)
			end
			self.tPatterns[i]:SetName(i)
			local m = (i - 1) % 3
			if m == 0 then nYOffset = nYOffset + 60 end
			self.tPatterns[i]:Move(nXOffset * m, nYOffset, 140, 40)
		end
	end
end

function StalkerResourceSystem:OnOptionsBarPatternHide(wndHandler, wndControl)
	wndControl:SetAnchorOffsets(0, 0, 0, 0)
end

function StalkerResourceSystem:OnPatternOverlayMouseEnter(wndHandler, wndControl, x, y)
	wndControl:GetParent():FindChild("HoverGlow"):SetOpacity(0.9, 10)
end

function StalkerResourceSystem:OnPatternOverlayMouseExit(wndHandler, wndControl, x, y)
	wndControl:GetParent():FindChild("HoverGlow"):SetOpacity(0, 2)
end

function StalkerResourceSystem:OnPatternOverlayMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self.bBarStyleStealthed then
		self.tPatterns[self.settings.tStealthed.nPatternID]:FindChild("SelectedGlow"):SetOpacity(0, 2)
		self.settings.tStealthed.nPatternID = tonumber(wndControl:GetParent():GetName())
		if self.bIsStealthed then self:SetStealthedPatternStyle() end
	else
		self.tPatterns[self.settings.nPatternID]:FindChild("SelectedGlow"):SetOpacity(0, 2)
		self.settings.nPatternID = tonumber(wndControl:GetParent():GetName())
	end
	wndControl:GetParent():FindChild("SelectedGlow"):SetOpacity(0.9, 7)
	self:SetPatternOverlayOptions()
end

function StalkerResourceSystem:OnPatternOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.fPatternOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:SetPatternOverlayOptions()
end

function StalkerResourceSystem:OnPatternShowButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bShowPattern = wndControl:IsChecked()
	self:SetPatternOverlayOptions()
end

function StalkerResourceSystem:OnPatternColorMouseButtonUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sPattern, false, UpdatePatternColor)
	end
end

function UpdatePatternColor()
	StalkerResourceSystem:SetPatternOverlayOptions()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sPattern = VILib:ConvertColorToHex(tColors.sPattern)
	OptionsWindowRef:FindChild("PatternColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sPattern)
end


---------------------------------------------------------------------------------------------------
-- OptionsBarBorder Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnOptionsBarBorderShow(wndHandler, wndControl)
	local nHeight = 70
	local nL, nT, nR, nB = wndControl:FindChild("BorderContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	nL, nT, nR, nB = wndControl:FindChild("GraphicBorderContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	wndControl:SetAnchorOffsets(0, 0, 0, nHeight)
end

function StalkerResourceSystem:OnOptionsBarBorderHide(wndHandler, wndControl)
	wndControl:SetAnchorOffsets(0, 0, 0, 0)
end

function StalkerResourceSystem:OnOpenGraphicBordersButtonUp(wndHandler, wndControl, eMouseButton)
	VILib:WriteToChat("Stop clicking me! There's none to select from yet - there's just the one for now.")
	Apollo.RegisterTimerHandler("VSRSSillyStuff", "IReallyDidThis", self)
	Apollo.CreateTimer("VSRSSillyStuff", 6, false)
	Apollo.StartTimer("VSRSSillyStuff")
end

function StalkerResourceSystem:IReallyDidThis()
	VILib:WriteToChat("Ok, you can click me again... It was actually kind of nice...")
end

function StalkerResourceSystem:OnGraphicBorderShowButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bShowGraphicBorder = wndControl:IsChecked()
	StalkerResourceSystem:SetGraphicBorder()
end

function StalkerResourceSystem:OnGraphicBorderTintMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sGraphicBorder, false, UpdateGraphicBorderColor)
	end
end

function UpdateGraphicBorderColor()
	StalkerResourceSystem:SetGraphicBorderColor()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sGraphicBorder = VILib:ConvertColorToHex(tColors.sGraphicBorder)
	OptionsWindowRef:FindChild("GraphicBorderColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sGraphicBorder)
end


---------------------------------------------------------------------------------------------------
-- OptionsBarFlash Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnOptionsBarFlashShow(wndHandler, wndControl)
	local nHeight = 60
	local nL, nT, nR, nB = wndControl:FindChild("BarFlashContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	wndControl:SetAnchorOffsets(0, 0, 0, nHeight)
end

function StalkerResourceSystem:OnOptionsBarFlashHide(wndHandler, wndControl)
	wndControl:SetAnchorOffsets(0, 0, 0, 0)
end


---------------------------------------------------------------------------------------------------
-- OptionsText Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnOptionsTextShow(wndHandler, wndControl)
	local nHeight = 60
	local nL, nT, nR, nB = wndControl:FindChild("TextContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	wndControl:SetAnchorOffsets(0, 0, 0, nHeight)
end

function StalkerResourceSystem:OnOptionsTextHide(wndHandler, wndControl)
	wndControl:SetAnchorOffsets(0, 0, 0, 0)
end


---------------------------------------------------------------------------------------------------
-- OptionsStealthed Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnOptionsStealthedShow(wndHandler, wndControl)
	local nHeight = 90
	local nL, nT, nR, nB = wndControl:FindChild("BarLookContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	nL, nT, nR, nB = wndControl:FindChild("BorderContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	nL, nT, nR, nB = wndControl:FindChild("TextContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	nL, nT, nR, nB = wndControl:FindChild("PatternContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	wndControl:SetAnchorOffsets(0, 0, 0, nHeight)
end

function StalkerResourceSystem:OnOptionsStealthedHide(wndHandler, wndControl)
	wndControl:SetAnchorOffsets(0, 0, 0, 0)
end

function StalkerResourceSystem:OpenBarStyleOptionsStealthed(wndHandler, wndControl, eMouseButton)
	self.bBarStyleStealthed = true
	self:ShowHideBarStyleOptions()
end

function StalkerResourceSystem:OnStealthedBarStyleButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bBarStyle = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedBarColorButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bBarColor = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedBarColorX2MouseUp(wndHandler, wndControl, eMouseButton)
	self.settings.tColors.bStealthedBar2x = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealtheBGBarColorButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bBGBarColor = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedMasterOpacityButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bMasterOpacity = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedHighlightOpacityButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bHighlightOpacity = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedBorderColorButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bBorderColor = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedBorderSizeButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bBorderSize = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedBorderOpacityButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bBorderOpacity = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedTextColorButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bTextColor = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedTextOpacityButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bTextOpacity = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedMasterOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tStealthed.fMasterOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedHighlightOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tStealthed.fHighlightOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedBorderSizeSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tStealthed.nBorderWidth = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedBorderOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tStealthed.fBorderOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedTextOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tStealthed.fTextOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedBarColorMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sStealthedBar, false, UpdateStealthedBarColor)
	end
end

function UpdateStealthedBarColor()
	StalkerResourceSystem:UpdateStealth()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sStealthedBar = VILib:ConvertColorToHex(tColors.sStealthedBar)
	OptionsWindowRef:FindChild("OptionsStealthed"):FindChild("BarLookContainer:BarColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sStealthedBar)
end

function StalkerResourceSystem:OnStealthedBGBarColorMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sStealthedBGBar, false, UpdateStealthedBGBarColor)
	end
end

function UpdateStealthedBGBarColor()
	StalkerResourceSystem:UpdateStealth()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sStealthedBGBar = VILib:ConvertColorToHex(tColors.sStealthedBGBar)
	OptionsWindowRef:FindChild("OptionsStealthed"):FindChild("BarLookContainer:BackgroundBarColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sStealthedBGBar)
end

function StalkerResourceSystem:OnStealthedBorderColorMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sStealthedBorder, false, UpdateStealthedBorderColor)
	end
end

function UpdateStealthedBorderColor()
	StalkerResourceSystem:UpdateStealth()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sStealthedBorder = VILib:ConvertColorToHex(tColors.sStealthedBorder)
	OptionsWindowRef:FindChild("OptionsStealthed"):FindChild("BorderContainer:BorderColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sStealthedBorder)
end

function StalkerResourceSystem:OnStealthedTextColorMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sStealthedText, false, UpdateStealthedTextColor)
	end
end

function UpdateStealthedTextColor()
	StalkerResourceSystem:UpdateStealth()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sStealthedText = VILib:ConvertColorToHex(tColors.sStealthedText)
	OptionsWindowRef:FindChild("OptionsStealthed"):FindChild("TextContainer:TextColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sStealthedText)
end

function StalkerResourceSystem:OnStealthedShowPatternButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bPattern = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedPatternOpacityButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bPatternOpacity = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedPatternColorButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.tStealthed.bPatternColor = wndControl:IsChecked()
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedPatternOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tStealthed.fPatternOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:UpdateStealth()
end

function StalkerResourceSystem:OnStealthedPatternColorMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if self:HasColorPicker() then
		ColorPicker.AdjustCColor(tColors.sStealthedPattern, false, UpdateStealthedPatternColor)
	end
end

function UpdateStealthedPatternColor()
	StalkerResourceSystem:UpdateStealth()
	if not StalkerResourceSystem.bOptionsOpen then return end
	SettingsRef.tColors.sStealthedPattern = VILib:ConvertColorToHex(tColors.sStealthedPattern)
	OptionsWindowRef:FindChild("OptionsStealthed"):FindChild("PatternContainer:PatternColor"):FindChild("ColorSwatch"):SetBGColor(SettingsRef.tColors.sStealthedPattern)
end


---------------------------------------------------------------------------------------------------
-- OptionsAdvanced Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnOptionsAdvancedShow(wndHandler, wndControl)
	local nHeight = 70
	local nL, nT, nR, nB = wndControl:FindChild("FadeTimesContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	nL, nT, nR, nB = wndControl:FindChild("SmoothUpdateContainer"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	wndControl:SetAnchorOffsets(0, 0, 0, nHeight)
end

function StalkerResourceSystem:OnOptionsAdvancedHide(wndHandler, wndControl)
	wndControl:SetAnchorOffsets(0, 0, 0, 0)
end


---------------------------------------------------------------------------------------------------
-- OptionsSpecials Functions
---------------------------------------------------------------------------------------------------

function StalkerResourceSystem:OnOptionsSpecialsShow( wndHandler, wndControl )
	local nHeight = 60
	local nL, nT, nR, nB = wndControl:FindChild("Below35Container"):GetAnchorOffsets()
	nHeight = nHeight + (nB - nT)
	wndControl:SetAnchorOffsets(0, 0, 0, nHeight)
end

function StalkerResourceSystem:OnOptionsSpecialsHide(wndHandler, wndControl)
	wndControl:SetAnchorOffsets(0, 0, 0, 0)
end



local StalkerResourceSystemInst = StalkerResourceSystem:new()
StalkerResourceSystem:Init()
